@@ -1,12 +1,17 @@
+"""Argparser factory for repod's CLI."""
+from __future__ import annotations
+
 import os
 from argparse import ArgumentParser, ArgumentTypeError
 from pathlib import Path
 
+from pydantic import AnyUrl, TypeAdapter, ValidationError
+
 from repod.common.enums import ArchitectureEnum
 
 
 class ArgParseFactory:
-    """A factory class to create different types of ArgumentParser instances
+    """A factory class to create different types of ArgumentParser instances.
 
     Attributes
     ----------
@@ -16,6 +21,13 @@ class ArgParseFactory:
     """
 
     def __init__(self, description: str = "default") -> None:
+        """Initialize an instance of ArgParseFactory.
+
+        Parameters
+        ----------
+        description: str
+            The description string of the resulting self.parser (an ArgumentParser)
+        """
         self.parser = ArgumentParser(description=description)
         self.parser.add_argument(
             "-c",
@@ -46,14 +58,13 @@ class ArgParseFactory:
 
     @classmethod
     def repod_file(cls) -> ArgumentParser:
-        """A class method to create an ArgumentParser for the repod-file script
+        """Create an ArgumentParser for the repod-file script.
 
         Returns
         -------
         ArgumentParser
             An ArgumentParser instance specific for the repod-file script
         """
-
         instance = cls(description="File actions for packages, management repository and sync databases.")
         subcommands = instance.parser.add_subparsers(dest="subcommand")
 
@@ -127,7 +138,7 @@ class ArgParseFactory:
 
         repo_importpkg_parser = repo_subcommands.add_parser(
             name="importpkg",
-            help="import packages of the same pkgbase to a repo",
+            help="import packages to a repo",
         )
         repo_importpkg_parser.add_argument(
             "file",
@@ -164,6 +175,14 @@ class ArgParseFactory:
             action="store_true",
             help="locate and use a signature file for each provided package file",
         )
+        repo_importpkg_parser.add_argument(
+            "-u",
+            "--source-url",
+            default=[],
+            nargs="+",
+            help="list of source URLs for added pkgbases (provided as one or more pkgbase=url strings)",
+            type=cls.string_to_tuple,
+        )
         mutual_exclusive_repo_importpkg = repo_importpkg_parser.add_mutually_exclusive_group()
         mutual_exclusive_repo_importpkg.add_argument(
             "-D",
@@ -239,7 +258,7 @@ class ArgParseFactory:
 
     @classmethod
     def string_to_writable_file_path(cls, input_: str) -> Path:
-        """Convert an input string into a Path to a file
+        """Convert an input string into a Path to a file.
 
         This method checks whether an (existing) file is writable. If the file does not exist the parent directory is
         checked for existence and whether it is writable.
@@ -259,7 +278,6 @@ class ArgParseFactory:
         Path
             A Path instance created from input_
         """
-
         path = Path(input_)
         if path.exists():
             if not path.is_file():
@@ -277,7 +295,7 @@ class ArgParseFactory:
 
     @classmethod
     def string_to_file_path(cls, input_: str) -> Path:
-        """Convert an input string into a Path to a file
+        """Convert an input string into a Path to a file.
 
         Parameters
         ----------
@@ -294,7 +312,6 @@ class ArgParseFactory:
         Path
             A Path instance created from input_
         """
-
         path = Path(input_)
         if not path.exists():
             raise ArgumentTypeError(f"the file '{input_}' does not exist")
@@ -304,7 +321,7 @@ class ArgParseFactory:
 
     @classmethod
     def string_to_dir_path(cls, input_: str) -> Path:
-        """Convert an input string into a Path to a directory
+        """Convert an input string into a Path to a directory.
 
         Parameters
         ----------
@@ -321,7 +338,6 @@ class ArgParseFactory:
         Path
             A Path instance created from input_
         """
-
         path = Path(input_)
         if not path.exists():
             raise ArgumentTypeError(f"the directory '{input_}' does not exist")
@@ -329,6 +345,44 @@ class ArgParseFactory:
             raise ArgumentTypeError(f"not a directory: {input_}")
         return path
 
+    @classmethod
+    def string_to_tuple(cls, input_: str) -> tuple[str, AnyUrl]:
+        """Convert an input string into a tuple of string and AnyUrl.
+
+        The input string is expected to be delimited by at least one "=" character to split on.
+
+        Parameters
+        ----------
+        input_: str
+            A string that is used to create a tuple
+
+        Raises
+        ------
+        ArgumentTypeError:
+            If input_ does not have at least one "=" to split on,
+            or if the resulting left-hand string contains a whitespace character after stripping,
+            or if the resulting right-hand string can not be validated as a URL.
+
+        Returns
+        -------
+        tuple[str, AnyUrl]
+            A tuple denoting a word and an instance of AnyUrl
+        """
+        try:
+            pkgbase, url_str = [str_.strip() for str_ in input_.split(sep="=", maxsplit=1)]
+        except ValueError:
+            raise ArgumentTypeError(f"There is no '=' in '{input_}'!") from None
+
+        if " " in pkgbase:
+            raise ArgumentTypeError(f"The string left of the '=' in '{input_}' must be a single word!")
+
+        try:
+            url = TypeAdapter(AnyUrl).validate_python(url_str)
+        except ValidationError:
+            raise ArgumentTypeError(f"The URL string '{url_str}' found in '{input_}' is invalid!") from None
+
+        return (pkgbase, url)
+
 
 def sphinx_repod_file() -> ArgumentParser:
     """Return ArgParseFactory.repod_file() for sphinx."""
