@@ -1,7 +1,7 @@
+"""Pydantic models shared throughout the codebase."""
 from __future__ import annotations
 
-import re
-from typing import List, Optional
+from pathlib import Path
 
 from email_validator import EmailNotValidError, validate_email
 from pydantic import (
@@ -11,28 +11,27 @@ from pydantic import (
     PositiveInt,
     conint,
     constr,
-    validator,
+    field_validator,
 )
 
 from repod.common.regex import (
     ARCHITECTURE,
     BASE64,
     EPOCH,
-    FILENAME,
     MD5,
+    OPTIONS,
+    PACKAGE_FILENAME,
     PACKAGE_NAME,
     PACKAGER_NAME,
     PKGREL,
-    RELATIVE_PATH,
     SHA256,
     VERSION,
 )
 from repod.version import alpm
-from repod.version.util import cmp
 
 
 class Arch(BaseModel):
-    """A model describing a single 'arch' attribute
+    """A model describing a single 'arch' attribute.
 
     Attributes
     ----------
@@ -41,24 +40,54 @@ class Arch(BaseModel):
         identifies a package's architecture
     """
 
-    arch: constr(regex=f"^{ARCHITECTURE}$")  # type: ignore[valid-type]  # noqa: F722
+    arch: constr(pattern=f"^{ARCHITECTURE}$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class Backup(BaseModel):
-    """A model describing a single 'backup' attribute
+    """A model describing a single 'backup' attribute.
 
     Attributes
     ----------
-    backup: Optional[List[str]]
+    backup: list[str] | None
         The attribute can be used to describe the (optional) data below a %BACKUP% identifier in a 'desc' file, which
         identifies which file(s) of a package pacman will create backups for
     """
 
-    backup: Optional[List[constr(regex=f"^{RELATIVE_PATH}$")]]  # type: ignore[valid-type]  # noqa: F722
+    backup: list[str] | None = None
+
+    @field_validator("backup")
+    @classmethod
+    def validate_backup(cls, backup: list[str] | None) -> list[str] | None:  # noqa: N805
+        """Validate the backup attribute.
+
+        The backup attribute may not contain strings that represent absolute Paths or Paths in the home directory
+
+        Parameters
+        ----------
+        backup: list[str] | None
+            An optional list of strings, representing paths to validate
+
+        Returns
+        -------
+        list[str] | None
+            None if backup is None, empty list if backup is empty list, or a validated list of strings
+        """
+        if backup:
+            for file in backup:
+                path = Path(file)
+                if path.is_absolute():
+                    raise ValueError(f"Absolute paths in a list of files are not valid: {path}")
+                parts = path.parts
+                if parts[0] == "home" and len(parts) > 1:
+                    raise ValueError(
+                        f"Files or directories in the home directory is not valid in a list of files: {path}"
+                    )
+
+        return backup
 
 
 class Base(BaseModel):
-    """A model describing a single 'base' attribute
+    """A model describing a single 'base' attribute.
 
     Attributes
     ----------
@@ -67,11 +96,11 @@ class Base(BaseModel):
         identifies a package's pkgbase
     """
 
-    base: constr(regex=f"^{PACKAGE_NAME}$")  # type: ignore[valid-type]  # noqa: F722
+    base: constr(pattern=f"^{PACKAGE_NAME}$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class BuildDate(BaseModel):
-    """A model describing a single 'builddate' attribute
+    """A model describing a single 'builddate' attribute.
 
     Attributes
     ----------
@@ -84,33 +113,33 @@ class BuildDate(BaseModel):
 
 
 class CheckDepends(BaseModel):
-    """A model describing a single 'checkdepends' attribute
+    """A model describing a single 'checkdepends' attribute.
 
     Attributes
     ----------
-    checkdepends: Optional[List[str]]
+    checkdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %CHECKDEPENDS% identifier in a 'desc' file,
         which identifies a package's checkdepends
     """
 
-    checkdepends: Optional[List[str]]
+    checkdepends: list[str] | None = None
 
 
 class Conflicts(BaseModel):
-    """A model describing a single 'conflicts' attribute
+    """A model describing a single 'conflicts' attribute.
 
     Attributes
     ----------
-    conflicts: Optional[List[str]]
+    conflicts: list[str] | None
         The attribute can be used to describe the (optional) data below a %CONFLICTS% identifier in a 'desc' file, which
         identifies what other package(s) a package conflicts with
     """
 
-    conflicts: Optional[List[str]]
+    conflicts: list[str] | None = None
 
 
 class CSize(BaseModel):
-    """A model describing a single 'csize' attribute
+    """A model describing a single 'csize' attribute.
 
     Attributes
     ----------
@@ -123,20 +152,20 @@ class CSize(BaseModel):
 
 
 class Depends(BaseModel):
-    """A model describing a single 'depends' attribute
+    """A model describing a single 'depends' attribute.
 
     Attributes
     ----------
-    depends: Optional[List[str]]
+    depends: list[str] | None
         The attribute can be used to describe the (optional) data below a %DEPENDS% identifier in a 'desc' file, which
         identifies what other package(s) a package depends on
     """
 
-    depends: Optional[List[str]]
+    depends: list[str] | None = None
 
 
 class Desc(BaseModel):
-    """A model describing a single 'desc' attribute
+    """A model describing a single 'desc' attribute.
 
     Attributes
     ----------
@@ -149,7 +178,7 @@ class Desc(BaseModel):
 
 
 class FileName(BaseModel):
-    """A model describing a single 'filename' attribute
+    """A model describing a single 'filename' attribute.
 
     Attributes
     ----------
@@ -158,46 +187,67 @@ class FileName(BaseModel):
         identifies a package's file name
     """
 
-    filename: constr(regex=f"^{FILENAME}$")  # type: ignore[valid-type]  # noqa: F722
+    filename: constr(pattern=f"^{PACKAGE_FILENAME}$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class FileList(BaseModel):
-    """A model describing an optional list of files
+    """A model describing an optional list of files.
 
     Attributes
     ----------
-    files: Optional[List[str]]
+    files: list[str] | None
         The attribute can be used to describe the (optional) data below a %FILES% identifier in a 'files' file, which
         identifies which file(s) belong to a package
     """
 
-    files: Optional[List[constr(regex=f"^{RELATIVE_PATH}$")]]  # type: ignore[valid-type]  # noqa: F722
+    files: list[str] | None = None
 
-    @validator("files")
-    def validate_no_file_in_home(cls, files: List[str]) -> Optional[List[str]]:
+    @field_validator("files")
+    @classmethod
+    def validate_files(cls, files: list[str] | None) -> list[str] | None:  # noqa: N805
+        """Validate the files attribute.
+
+        The files attribute may not contain strings that represent absolute Paths or Paths in the home directory
+
+        Parameters
+        ----------
+        files: list[str] | None
+            An optional list of strings, representing paths to validate
+
+        Returns
+        -------
+        list[str] | None
+            None if files is None, empty list if files is empty list, or a validated list of strings
+        """
         if files:
             for file in files:
-                if re.search("^(home/).+$", file):
-                    raise ValueError("A package must not provide files in /home")
+                path = Path(file)
+                if path.is_absolute():
+                    raise ValueError(f"Absolute paths in a list of files are not valid: {path}")
+                parts = path.parts
+                if parts[0] == "home" and len(parts) > 1:
+                    raise ValueError(
+                        f"Files or directories in the home directory is not valid in a list of files: {path}"
+                    )
 
         return files
 
 
 class Groups(BaseModel):
-    """A model describing a single 'groups' attribute
+    """A model describing a single 'groups' attribute.
 
     Attributes
     ----------
-    groups: Optional[List[str]]
+    groups: list[str] | None
         The attribute can be used to describe the (optional) data below a %GROUPS% identifier in a 'desc' file, which
         identifies a package's groups
     """
 
-    groups: Optional[List[constr(regex=f"^{PACKAGE_NAME}$")]]  # type: ignore[valid-type]  # noqa: F722
+    groups: list[constr(pattern=f"^{PACKAGE_NAME}$")] | None = None  # type: ignore[valid-type]  # noqa: F722
 
 
 class ISize(BaseModel):
-    """A model describing a single 'isize' attribute
+    """A model describing a single 'isize' attribute.
 
     Attributes
     ----------
@@ -210,33 +260,33 @@ class ISize(BaseModel):
 
 
 class License(BaseModel):
-    """A model describing a single 'license' attribute
+    """A model describing a single 'license' attribute.
 
     Attributes
     ----------
-    license: List[str]
+    license: list[str]
         The attribute can be used to describe the (required) data below a %LICENSE% identifier in a 'desc' file, which
         identifies a package's license(s)
     """
 
-    license: List[str]
+    license: list[str]  # noqa: A003
 
 
 class MakeDepends(BaseModel):
-    """A model describing a single 'makedepends' attribute
+    """A model describing a single 'makedepends' attribute.
 
     Attributes
     ----------
-    makedepends: Optional[List[str]]
+    makedepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %MAKEDEPENDS% identifier in a 'desc' file,
         which identifies a package's makedepends
     """
 
-    makedepends: Optional[List[str]]
+    makedepends: list[str] | None = None
 
 
 class Md5Sum(BaseModel):
-    """A model describing a single 'md5sum' attribute
+    """A model describing a single 'md5sum' attribute.
 
     Attributes
     ----------
@@ -245,11 +295,11 @@ class Md5Sum(BaseModel):
         identifies a package's md5 checksum
     """
 
-    md5sum: constr(regex=MD5)  # type: ignore[valid-type]
+    md5sum: constr(pattern=MD5)  # type: ignore[valid-type]
 
 
 class Name(BaseModel):
-    """A model describing a single 'name' attribute
+    """A model describing a single 'name' attribute.
 
     Attributes
     ----------
@@ -258,11 +308,25 @@ class Name(BaseModel):
         identifies a package's name
     """
 
-    name: constr(regex=f"^{PACKAGE_NAME}$")  # type: ignore[valid-type]  # noqa: F722
+    name: constr(pattern=f"^{PACKAGE_NAME}$")  # type: ignore[valid-type]  # noqa: F722
+
+
+class Options(BaseModel):
+    """A list of makepkg.conf OPTIONS used during the creation of a package.
+
+    For valid values refer to the OPTIONS subsection in https://man.archlinux.org/man/makepkg.conf.5#OPTIONS
+
+    Attributes
+    ----------
+    options: list[str] | None
+        An optional list of strings representing makepkg.conf OPTIONS used during the creation of a package
+    """
+
+    options: list[constr(pattern=rf"^{OPTIONS}$")] | None = None  # type: ignore[valid-type]  # noqa: F722
 
 
 class Packager(BaseModel):
-    """A model describing a single 'packager' attribute
+    """A model describing a single 'packager' attribute.
 
     Attributes
     ----------
@@ -271,22 +335,39 @@ class Packager(BaseModel):
         identifies a package's packager
     """
 
-    packager: constr(regex=(rf"^{PACKAGER_NAME}\s<(.*)>$"))  # type: ignore[valid-type]  # noqa: F722
+    packager: constr(pattern=(rf"^{PACKAGER_NAME}\s<(.*)>$"))  # type: ignore[valid-type]  # noqa: F722
 
-    @validator("packager")
-    def validate_packager_has_valid_email(cls, packager: str) -> str:
+    @field_validator("packager")
+    @classmethod
+    def validate_packager_has_valid_email(cls, packager: str) -> str:  # noqa: N805
+        """Validate that Packager has an email in the UID.
 
+        Parameters
+        ----------
+        packager: str
+            The packager UID string
+
+        Raises
+        ------
+        ValuError
+            If no valid mail is found in packager
+
+        Returns
+        -------
+        str
+            A validated Packager UID string
+        """
         email = packager.replace(">", "").split("<")[1]
         try:
             validate_email(email, check_deliverability=False)
         except EmailNotValidError as e:
-            raise ValueError(f"The packager email is not valid: {email}\n{e}")
+            raise ValueError(f"The packager email is not valid: {email}\n{e}") from e
 
         return packager
 
 
 class PgpSig(BaseModel):
-    """A model describing a single 'pgpsig' attribute
+    """A model describing a single 'pgpsig' attribute.
 
     Attributes
     ----------
@@ -295,37 +376,77 @@ class PgpSig(BaseModel):
         identifies a package's PGP signature
     """
 
-    pgpsig: Optional[constr(regex=f"^{BASE64}$")]  # type: ignore[valid-type]  # noqa: F722
+    pgpsig: constr(pattern=f"^{BASE64}$") | None = None  # type: ignore[valid-type]  # noqa: F722
 
 
-class Provides(BaseModel):
-    """A model describing a single 'provides' attribute
+class PkgBase(BaseModel):
+    """A pkgbase for a package.
+
+    Refer to https://man.archlinux.org/man/PKGBUILD.5.en#PACKAGE_SPLITTING for details on pkgbase
 
     Attributes
     ----------
-    provides: Optional[List[str]]
+    pkgbase: str
+        A string representing the pkgbase of a package
+    """
+
+    pkgbase: constr(pattern=rf"^{PACKAGE_NAME}$")  # type: ignore[valid-type]  # noqa: F722
+
+
+class PkgDesc(BaseModel):
+    """A model describing a single pkgdesc attribute.
+
+    Attributes
+    ----------
+    pkgdesc: str
+        A string used as package description
+    """
+
+    pkgdesc: str
+
+
+class PkgName(BaseModel):
+    """A pkgname of a package.
+
+    Refer to the pkgname section in https://man.archlinux.org/man/PKGBUILD.5.en#OPTIONS_AND_DIRECTIVES for details
+
+    Attributes
+    ----------
+    pkgname: str
+        A string representing the pkgname of a package
+    """
+
+    pkgname: constr(pattern=rf"^{PACKAGE_NAME}$")  # type: ignore[valid-type]  # noqa: F722
+
+
+class Provides(BaseModel):
+    """A model describing a single 'provides' attribute.
+
+    Attributes
+    ----------
+    provides: list[str] | None
         The attribute can be used to describe the (optional) data below a %PROVIDES% identifier in a 'desc' file, which
         identifies what other package(s) a package provides
     """
 
-    provides: Optional[List[str]]
+    provides: list[str] | None = None
 
 
 class Replaces(BaseModel):
-    """A model describing a single 'replaces' attribute
+    """A model describing a single 'replaces' attribute.
 
     Attributes
     ----------
-    replaces: Optional[List[str]]
+    replaces: list[str] | None
         The attribute can be used to describe the (optional) data below a %REPLACES% identifier in a 'desc' file, which
         identifies what other package(s) a package replaces
     """
 
-    replaces: Optional[List[str]]
+    replaces: list[str] | None = None
 
 
 class SchemaVersionV1(BaseModel):
-    """A model describing a schema version 1
+    """A model describing a schema version 1.
 
     Attributes
     ----------
@@ -337,7 +458,7 @@ class SchemaVersionV1(BaseModel):
 
 
 class SchemaVersionV2(BaseModel):
-    """A model describing a schema version 2
+    """A model describing a schema version 2.
 
     Attributes
     ----------
@@ -349,7 +470,7 @@ class SchemaVersionV2(BaseModel):
 
 
 class Sha256Sum(BaseModel):
-    """A model describing a single 'sha256sum' attribute
+    """A model describing a single 'sha256sum' attribute.
 
     Attributes
     ----------
@@ -358,24 +479,24 @@ class Sha256Sum(BaseModel):
         which identifies a package's sha256 checksum
     """
 
-    sha256sum: constr(regex=SHA256)  # type: ignore[valid-type]
+    sha256sum: constr(pattern=SHA256)  # type: ignore[valid-type]
 
 
 class OptDepends(BaseModel):
-    """A model describing a single 'optdepends' attribute
+    """A model describing a single 'optdepends' attribute.
 
     Attributes
     ----------
-    optdepends: Optional[List[str]]
+    optdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %OPTDEPENDS% identifier in a 'desc' file,
         which identifies what other package(s) a package optionally depends on
     """
 
-    optdepends: Optional[List[str]]
+    optdepends: list[str] | None = None
 
 
 class Url(BaseModel):
-    """A model describing a single 'url' attribute
+    """A model describing a single 'url' attribute.
 
     Attributes
     ----------
@@ -387,122 +508,62 @@ class Url(BaseModel):
     url: HttpUrl
 
 
+class SourceUrl(BaseModel):
+    """A URL pointing at sources.
+
+    Attributes
+    ----------
+    source_url: HttpUrl | None
+        An optional url that points at sources (defaults to None)
+    """
+
+    source_url: HttpUrl | None = None
+
+
 class Epoch(BaseModel):
-    """A model dscribing a single 'epoch' attribute
+    """A model dscribing a single 'epoch' attribute.
 
     The epoch denotes a downgrade in version of a given package (a version with an epoch trumps one without)
 
     Attributes
     ----------
-    epoch: PositiveInt
-        A string representing a valid epoch of a package
+    epoch: PositiveInt | None
+        An optional positive integer representing the epoch of a package
     """
 
-    epoch: PositiveInt
-
-    def vercmp(self, epoch: Epoch) -> int:
-        """Compare the epoch with another
-
-        The comparison algorithm is based on pyalpm's/ pacman's vercmp behavior.
-
-        Returns
-        -------
-        int
-            -1 if self.epoch is older than epoch
-            0 if self.epoch is equal to epoch
-            1 if self.epoch is newer than epoch
-        """
-
-        return cmp(self.epoch, epoch.epoch)
+    epoch: PositiveInt | None = None
 
 
 class PkgRel(BaseModel):
-    """A model dscribing a single 'pkgrel' attribute
+    """A model dscribing a single 'pkgrel' attribute.
 
     The pkgrel denotes the build version of a given package
 
     Attributes
     ----------
     pkgrel: str
-        A string representing a valid pkgrel of a package
+        A string representing the pkgrel (package release version) of a package
     """
 
-    pkgrel: constr(regex=rf"^{PKGREL}$")  # type: ignore[valid-type]  # noqa: F722
-
-    def as_list(self) -> List[str]:
-        """Return the pkgrel components as list
-
-        The version string is split by "."
-
-        Returns
-        -------
-        List[str]
-            A list of strings representing the components of the pkgrel
-        """
-
-        return [str(part) for part in self.pkgrel.split(".")]
-
-    def vercmp(self, pkgrel: PkgRel) -> int:
-        """Compare the pkgrel with another
-
-        The comparison algorithm is based on pyalpm's/ pacman's vercmp behavior.
-
-        Returns
-        -------
-        int
-            -1 if self.pkgrel is older than pkgrel
-            0 if self.pkgrel is equal to pkgrel
-            1 if self.pkgrel is newer than pkgrel
-        """
-
-        return alpm.vercmp(self.pkgrel, pkgrel.pkgrel)
+    pkgrel: constr(pattern=rf"^{PKGREL}$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class PkgVer(BaseModel):
-    """A model dscribing a single 'pkgver' attribute
+    """A model dscribing a single 'pkgver' attribute.
 
     The pkgver denotes the upstream version of a given package
 
     Attributes
     ----------
     pkgver: str
-        A string representing a valid pkgver of a package
+        A string representing the pkgver (upstream package version) of a package
     """
 
-    pkgver: constr(regex=rf"^({VERSION})$")  # type: ignore[valid-type]  # noqa: F722
-
-    def as_list(self) -> List[str]:
-        """Return the pkgver components as list
-
-        The version string is split on none alpanum characters
-
-        Returns
-        -------
-        List[str]
-            A list of strings representing the components of the pkgver
-        """
-
-        return [part for part in re.split(r"[^a-zA-Z\d]", self.pkgver) if part]
-
-    def vercmp(self, pkgver: PkgVer) -> int:
-        """Compare the pkgver with another
-
-        The comparison algorithm is based on pyalpm's/ pacman's vercmp behavior.
-        If PYALPM_VERCMP is True, pyalpm has been imported and its implementation of vercmp() is used.
-
-        Returns
-        -------
-        int
-            -1 if self.pkgver is older than pkgver
-            0 if self.pkgver is equal to pkgver
-            1 if self.pkgver is newer than pkgver
-        """
-
-        return alpm.vercmp(self.pkgver, pkgver.pkgver)
+    pkgver: constr(pattern=rf"^({VERSION})$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class Version(BaseModel):
-    """A model describing a single 'version' attribute
+    """A model describing a single 'version' attribute.
 
     Attributes
     ----------
@@ -511,48 +572,45 @@ class Version(BaseModel):
         identifies a package's version (this is the accumulation of epoch, pkgver and pkgrel)
     """
 
-    version: constr(regex=rf"^({EPOCH}|){VERSION}-{PKGREL}$")  # type: ignore[valid-type]  # noqa: F722
+    version: constr(pattern=rf"^({EPOCH}|){VERSION}-{PKGREL}$")  # type: ignore[valid-type]  # noqa: F722
 
-    def get_epoch(self) -> Optional[Epoch]:
-        """Return the epoch of the version
+    def get_epoch(self: Version) -> Epoch | None:
+        """Return the epoch of the version.
 
         Returns
         -------
-        Optional[int]
+        int | None
             An optional string representing the epoch of the version
         """
-
         if ":" in self.version:
             return Epoch(epoch=self.version.split(":")[0])
-        else:
-            return None
 
-    def get_pkgver(self) -> PkgVer:
-        """Return the pkgver of the version
+        return None
+
+    def get_pkgver(self: Version) -> PkgVer:
+        """Return the pkgver of the version.
 
         Returns
         -------
         PkgVer
             A PkgVer representing the pkgver of the version
         """
-
         pkgver_pkgrel = self.version.split(":")[1] if ":" in self.version else self.version
         return PkgVer(pkgver=str(pkgver_pkgrel.split("-")[0]))
 
-    def get_pkgrel(self) -> PkgRel:
-        """Return the pkgrel of the version
+    def get_pkgrel(self: Version) -> PkgRel:
+        """Return the pkgrel of the version.
 
         Returns
         -------
         PkgRel
             A PkgRel representing the pkgrel of the version
         """
-
         pkgver_pkgrel = self.version.split(":")[1] if ":" in self.version else self.version
         return PkgRel(pkgrel=str(pkgver_pkgrel.split("-")[1]))
 
-    def vercmp(self, version: Version) -> int:
-        """Compare the version with another
+    def vercmp(self: Version, version: Version) -> int:
+        """Compare the version with another.
 
         The comparison algorithm is based on pyalpm's/ pacman's vercmp behavior.
         If PYALPM_VERCMP is True, pyalpm has been imported and its implementation of vercmp() is used.
@@ -564,11 +622,10 @@ class Version(BaseModel):
             0 if self.version is equal to version
             1 if self.version is newer than version
         """
-
         return alpm.pkg_vercmp(self.version, version.version)
 
-    def is_older_than(self, version: Version) -> bool:
-        """Check whether the version is older than a provided version
+    def is_older_than(self: Version, version: Version) -> bool:
+        """Check whether the version is older than a provided version.
 
         Parameters
         ----------
@@ -579,11 +636,10 @@ class Version(BaseModel):
         -------
         True if self.version is older than the provided version, False otherwise.
         """
-
         return True if self.vercmp(version=version) < 0 else False
 
-    def is_newer_than(self, version: Version) -> bool:
-        """Check whether the version is newer than a provided version
+    def is_newer_than(self: Version, version: Version) -> bool:
+        """Check whether the version is newer than a provided version.
 
         Parameters
         ----------
@@ -594,5 +650,4 @@ class Version(BaseModel):
         -------
         True if self.version is newer than the provided version, False otherwise.
         """
-
         return True if self.vercmp(version=version) > 0 else False
