@@ -1,3 +1,4 @@
+"""Common function and tools to work with files."""
 from gzip import BadGzipFile
 from gzip import open as gzip_open
 from io import BytesIO, StringIO
@@ -5,7 +6,7 @@ from logging import debug
 from pathlib import Path
 from tarfile import ReadError, TarFile
 from tarfile import open as tarfile_open
-from typing import IO, Dict, List, Literal, Optional, Set, Union
+from typing import IO, Literal
 
 import magic
 from pyzstd import CParameter, ZstdDict, ZstdFile
@@ -15,16 +16,17 @@ from repod.errors import RepoManagementFileError, RepoManagementFileNotFoundErro
 
 
 class ZstdTarFile(TarFile):
-    """A class to provide reading and writing of zstandard files using TarFile functionality"""
+    """A class to provide reading and writing of zstandard files using TarFile functionality."""
 
     def __init__(  # type: ignore[no-untyped-def]
         self,
-        name: Union[str, Path],
+        name: str | Path,
         mode: Literal["r", "a", "w", "x"] = "r",
-        level_or_option: Union[None, int, Dict[CParameter, int]] = None,
-        zstd_dict: Optional[ZstdDict] = None,
+        level_or_option: None | int | dict[CParameter, int] = None,
+        zstd_dict: ZstdDict | None = None,
         **kwargs,
     ) -> None:
+        """Initialize an instance of ZstdTarFile."""
         self.zstd_file = ZstdFile(
             filename=name,
             mode=mode,
@@ -39,6 +41,7 @@ class ZstdTarFile(TarFile):
             raise RepoManagementFileError(f"An error occured while trying to open the file {name}!\n{e}")
 
     def close(self) -> None:
+        """Close the file."""
         try:
             super().close()
         finally:
@@ -46,7 +49,7 @@ class ZstdTarFile(TarFile):
 
 
 def compression_type_of_tarfile(path: Path) -> CompressionTypeEnum:
-    """Retrieve the compression type of a tar file
+    """Retrieve the compression type of a tar file.
 
     Parameters
     ----------
@@ -63,43 +66,42 @@ def compression_type_of_tarfile(path: Path) -> CompressionTypeEnum:
     CompressionTypeEnum
         A member of CompressionTypeEnum, that reflects the compression type of tar file at path
     """
-
     with open(path, "rb") as f:
         file_start_bytes: bytes = f.read(2048)
 
     # Try and detect the instance of the libmagic shared library (loaded via
     # ctypes) used by the magic.py shipped with file.
     if hasattr(magic, "_libraries"):  # pragma: no cover
-        file = magic.detect_from_content(file_start_bytes).name  # type: ignore[attr-defined]
+        types = magic.detect_from_content(file_start_bytes).name  # type: ignore[attr-defined]
     else:
-        file = magic.from_buffer(file_start_bytes)
-    file = " ".join(file.split()[0:3]).lower().strip(",")
-    debug(f"Type of file {path} detected as: {file}")
+        m = magic.Magic(keep_going=True)
+        types = m.from_buffer(file_start_bytes)
+    types = types.lower().strip(",")
+    debug(f"Types of file {path} detected as: {types}")
 
-    match file:
-        case "posix tar archive":
-            return CompressionTypeEnum.NONE
-        case "bzip2 compressed data":
-            return CompressionTypeEnum.BZIP2
-        case "gzip compressed data":
-            return CompressionTypeEnum.GZIP
-        case "xz compressed data":
-            return CompressionTypeEnum.LZMA
-        case "zstandard compressed data":
-            return CompressionTypeEnum.ZSTANDARD
-        case _:
-            raise RepoManagementFileError(
-                f"An error occured while attempting to retrieve the compression type of tar file: {path}!\n"
-                "Unknown compression type encountered."
-            )
+    if "posix tar archive" in types:
+        return CompressionTypeEnum.NONE
+    elif "bzip2 compressed data" in types:
+        return CompressionTypeEnum.BZIP2
+    elif "gzip compressed data" in types:
+        return CompressionTypeEnum.GZIP
+    elif "xz compressed data" in types:
+        return CompressionTypeEnum.LZMA
+    elif "zstandard compressed data" in types:
+        return CompressionTypeEnum.ZSTANDARD
+    else:
+        raise RepoManagementFileError(
+            f"An error occured while attempting to retrieve the compression type of tar file: {path}!\n"
+            "Unknown compression type encountered."
+        )
 
 
 def open_tarfile(
     path: Path,
-    compression: Optional[CompressionTypeEnum] = None,
+    compression: CompressionTypeEnum | None = None,
     mode: Literal["r", "w", "x"] = "r",
 ) -> TarFile:
-    """Open a file as a TarFile
+    """Open a file as a TarFile.
 
     This function distinguishes between bzip2, gzip, lzma and zstandard compression depending on file suffix.
     The detection can be overridden by providing either a file suffix or compression type.
@@ -108,7 +110,7 @@ def open_tarfile(
     ----------
     path: Path
         A Path to a file
-    compression: Optional[CompressionTypeEnum]
+    compression: CompressionTypeEnum | None
         An optional compression type to override the detection based on mime type.
     mode: Literal["r", "w", "x"]
         A mode to open the file with (defaults to "r").
@@ -130,7 +132,6 @@ def open_tarfile(
     tarfile.Tarfile
         An instance of Tarfile
     """
-
     debug(f"Opening file {path}...")
 
     if not path.is_absolute():
@@ -162,8 +163,8 @@ async def extract_file_from_tarfile(
     file: str,
     as_stringio: bool = False,
     gzip_compressed: bool = False,
-) -> Union[IO[bytes], StringIO]:
-    """Extract a file from a TarFile and return it as a bytes stream or text I/O buffer
+) -> IO[bytes] | StringIO:
+    """Extract a file from a TarFile and return it as a bytes stream or text I/O buffer.
 
     Parameters
     ----------
@@ -185,10 +186,9 @@ async def extract_file_from_tarfile(
 
     Returns
     -------
-    IO[bytes]
-        A bytes stream that represents the file to extract
+    IO[bytes] | StringIO
+        A bytes stream or StringIO that represents the file to extract
     """
-
     debug(f"Extracting file {file} from {str(tarfile.name)}...")
 
     try:
@@ -216,14 +216,14 @@ async def extract_file_from_tarfile(
             return extracted
 
 
-def names_in_tarfile(tarfile: TarFile, names: Union[List[str], Set[str]]) -> bool:
-    """Check whether a list of names is found in the list of names of a TarFile
+def names_in_tarfile(tarfile: TarFile, names: list[str] | set[str]) -> bool:
+    """Check whether a list of names is found in the list of names of a TarFile.
 
     Parameters
     ----------
     tarfile: TarFile
         A TarFile to lookup names in
-    names: Union[List[str], Set[str]]
+    names: list[str] | set[str]
         A list or set of names to lookup
 
     Returns
@@ -231,7 +231,6 @@ def names_in_tarfile(tarfile: TarFile, names: Union[List[str], Set[str]]) -> boo
     bool
         True if all names can be found in the list of TarFile names, else False
     """
-
     if isinstance(names, list):
         names = set(names)
 
@@ -239,3 +238,30 @@ def names_in_tarfile(tarfile: TarFile, names: Union[List[str], Set[str]]) -> boo
         return True
     else:
         return False
+
+
+def read_text_from_file(path: str | Path) -> StringIO:
+    """Read text from a file and return it in a StringIO.
+
+    Parameters
+    ----------
+    path: str | Path
+        A Path or str representing a file to read
+
+    Raises
+    ------
+    RepoManagementFileNotFoundError
+        If the file identifed by path does not exist
+
+    Returns
+    -------
+    StringIO
+        A string IO stream representing the contents of the file
+    """
+    if isinstance(path, str):
+        path = Path(path)
+
+    try:
+        return StringIO(initial_value=path.read_text())
+    except FileNotFoundError as e:
+        raise RepoManagementFileNotFoundError(e)
