@@ -1,11 +1,12 @@
+"""Handling of .MTREE files."""
 from __future__ import annotations
 
 import io
 import re
 from logging import debug
 from pathlib import Path
-from typing import Dict, List, Optional, Union
 
+from orjson import OPT_APPEND_NEWLINE, OPT_INDENT_2, OPT_SORT_KEYS, dumps
 from pydantic import (
     BaseModel,
     NonNegativeFloat,
@@ -21,7 +22,7 @@ from repod.errors import RepoManagementValidationError
 
 
 class SystemGID(BaseModel):
-    """The group ID of a system group
+    """The group ID of a system group.
 
     Attributes
     ----------
@@ -33,33 +34,33 @@ class SystemGID(BaseModel):
 
 
 class LinkTarget(BaseModel):
-    """The target location of a symlink
+    """The target location of a symlink.
 
     Attributes
     ----------
-    link: Optional[str]
+    link: str | None
         An optional string representing a relative or absolute file
     """
 
-    link: Optional[  # type: ignore[valid-type]
-        constr(regex=rf"^({RELATIVE_MTREE_PATH}|{ABSOLUTE_MTREE_PATH})$")  # noqa: F722
-    ]
+    link: constr(  # type: ignore[valid-type]
+        pattern=rf"^({RELATIVE_MTREE_PATH}|{ABSOLUTE_MTREE_PATH})$"  # noqa: F722
+    ) | None = None
 
 
 class Md5(BaseModel):
-    """An MD5 checksum
+    """An MD5 checksum.
 
     Attributes
     ----------
-    md5: Optional[str]
+    md5: str | None
         An optional string representing an MD5 checksum
     """
 
-    md5: Optional[constr(regex=rf"^{MD5}$")]  # type: ignore[valid-type]  # noqa: F722
+    md5: constr(pattern=rf"^{MD5}$") | None = None  # type: ignore[valid-type]  # noqa: F722
 
 
 class FileMode(BaseModel):
-    """A numeric unix file mode
+    """A numeric unix file mode.
 
     Attributes
     ----------
@@ -67,11 +68,11 @@ class FileMode(BaseModel):
         A three or four digit long string, consisting only of valid file modes
     """
 
-    mode: constr(regex=r"^[01234567]{3,4}$")  # type: ignore[valid-type]  # noqa: F722
+    mode: constr(pattern=r"^[01234567]{3,4}$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class MTreeEntryName(BaseModel):
-    """A file name in mtree representation
+    """A file name in mtree representation.
 
     The mtree format allows for encoding characters using a block of backslash and three octal digits (see
     https://man.archlinux.org/man/mtree.5#General_Format).
@@ -82,11 +83,11 @@ class MTreeEntryName(BaseModel):
         A string representing an absolute file location in mtree format
     """
 
-    name: constr(regex=rf"^{ABSOLUTE_MTREE_PATH}$")  # type: ignore[valid-type]  # noqa: F722
+    name: constr(pattern=rf"^{ABSOLUTE_MTREE_PATH}$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class Sha256(BaseModel):
-    """A SHA-256 checksum
+    """A SHA-256 checksum.
 
     Attributes
     ----------
@@ -94,11 +95,11 @@ class Sha256(BaseModel):
         An optional string representing a SHA-256 checksum
     """
 
-    sha256: Optional[constr(regex=rf"^{SHA256}$")]  # type: ignore[valid-type]  # noqa: F722
+    sha256: constr(pattern=rf"^{SHA256}$") | None = None  # type: ignore[valid-type]  # noqa: F722
 
 
 class FileSize(BaseModel):
-    """A file size in bytes
+    """A file size in bytes.
 
     Attributes
     ----------
@@ -106,11 +107,11 @@ class FileSize(BaseModel):
         A non-negative integer describing a file size in bytes
     """
 
-    size: Optional[NonNegativeInt]
+    size: NonNegativeInt | None = None
 
 
 class UnixTime(BaseModel):
-    """A timestamp in seconds since the epoch
+    """A timestamp in seconds since the epoch.
 
     Attributes
     ----------
@@ -122,7 +123,7 @@ class UnixTime(BaseModel):
 
 
 class MTreeEntryType(BaseModel):
-    """A file type for mtree entries (see https://man.archlinux.org/man/mtree.5#Keywords)
+    """A file type for mtree entries (see https://man.archlinux.org/man/mtree.5#Keywords).
 
     Attributes
     ----------
@@ -130,11 +131,11 @@ class MTreeEntryType(BaseModel):
         A string representing a valid mtree type (one of block, char, dir, fifo, file, link or socket)
     """
 
-    type_: constr(regex=r"^(block|char|dir|fifo|file|link|socket)$")  # type: ignore[valid-type]  # noqa: F722
+    type_: constr(pattern=r"^(block|char|dir|fifo|file|link|socket)$")  # type: ignore[valid-type]  # noqa: F722
 
 
 class SystemUID(BaseModel):
-    """The user ID of a system user
+    """The user ID of a system user.
 
     Attributes
     ----------
@@ -146,13 +147,13 @@ class SystemUID(BaseModel):
 
 
 class MTreeEntry(BaseModel):
-    """An entry in an MTree
+    """An entry in an MTree.
 
     This is a template class and should not be used directly. Instead instantiate one of the classes derived from it.
     """
 
     def get_file_path(self) -> Path:
-        """Return the file name as a Path
+        """Return the file name as a Path.
 
         The mtree format allows for encoding characters using a block of backslash and three octal digits (see
         https://man.archlinux.org/man/mtree.5#General_Format).
@@ -168,20 +169,19 @@ class MTreeEntry(BaseModel):
         Path
             The Path representation of name
         """
-
         if not hasattr(self, "name"):
             raise RuntimeError("It is not possible to retrieve a file path from the template class MTreeEntry!")
 
-        output_name = self.name  # type: ignore[attr-defined]
+        output_name = self.name
 
         if len(re.findall(r"\\[0-9A-F]{3}", output_name)) > 0:
             output_name = output_name.encode("latin1").decode("unicode-escape").encode("latin1").decode("utf8")
-            debug(f"Converted MTree path {self.name} to {output_name}.")  # type: ignore[attr-defined]
+            debug(f"Converted MTree path {self.name} to {output_name}.")
 
         return Path(output_name)
 
-    def get_link_path(self, resolve: bool = False) -> Optional[Path]:
-        """Return the link as a Path
+    def get_link_path(self, resolve: bool = False) -> Path | None:
+        """Return the link as a Path.
 
         The mtree format allows for encoding characters using a block of backslash and three octal digits (see
         https://man.archlinux.org/man/mtree.5#General_Format).
@@ -200,14 +200,13 @@ class MTreeEntry(BaseModel):
 
         Returns
         -------
-        Optional[Path]
+        Path | None
             The Path representation of link, or None if there is None
         """
-
         if not hasattr(self, "link"):
             raise RuntimeError("It is not possible to retrieve a file path from the template class MTreeEntry!")
 
-        output_name = self.link  # type: ignore[attr-defined]
+        output_name = self.link
         if output_name is None:
             return output_name
 
@@ -226,7 +225,7 @@ class MTreeEntry(BaseModel):
                 return (self.get_file_path() / ".." / output_path).resolve()
 
     def get_type(self) -> str:
-        """Return the type as a string
+        """Return the type as a string.
 
         Raises
         ------
@@ -238,11 +237,10 @@ class MTreeEntry(BaseModel):
         str
             The type of the MTreeEntry
         """
-
         if not hasattr(self, "type_"):
             raise RuntimeError("It is not possible to retrieve a type from the template class MTreeEntry!")
 
-        return str(self.type_)  # type: ignore[attr-defined]
+        return str(self.type_)
 
 
 class MTreeEntryV1(
@@ -259,15 +257,15 @@ class MTreeEntryV1(
     SystemGID,
     SystemUID,
 ):
-    """An entry in an MTree (version 1)
+    """An entry in an MTree (version 1).
 
     Attributes
     ----------
     gid: int
         A group ID >0, <1000
-    link: Optional[str]
+    link: str | None
         An optional string representing a relative or absolute file
-    md5: Optional[str]
+    md5: str | None
         A optional string representing an MD5 checksum
     mode: str
         A three or four digit long string, consisting only of valid file modes
@@ -291,19 +289,19 @@ class MTreeEntryV1(
 
 
 class MTree(BaseModel):
-    """A class to describe an mtree file
+    """A class to describe an mtree file.
 
     Attributes
     ----------
-    files: List[File]
+    files: list[File]
         A list of File instances, representing the entries in an mtree file
     """
 
-    entries: List[MTreeEntry]
+    entries: list[MTreeEntry]
 
     @classmethod
     def from_file(cls, data: io.StringIO) -> MTree:
-        """Create an instance of MTree from an io.StringIO representing the contents of an mtree file
+        """Create an instance of MTree from an io.StringIO representing the contents of an mtree file.
 
         Parameters
         ----------
@@ -321,55 +319,66 @@ class MTree(BaseModel):
             An instance of MTree, derived from data
         """
 
-        base_settings: Dict[str, Union[int, str]] = {}
-        entries: List[MTreeEntry] = []
+        def sanitize_mtree_pairs(
+            settings_list: list[list[str]],
+            settings_dict: dict[str, float | int | str],
+        ) -> None:
+            """Sanitize mtree pairs in a list and add them to a dict.
+
+            Parameters
+            ----------
+            settings_list: list[list[str]]
+                A list of string lists, that represent mtree key-value pairs
+            settings_dict: dict[str, float | int | str]
+                A dict to which sanitized mtree key-value pairs are added
+            """
+            for setting in settings_list:
+                settings_key = setting[0]
+                setting_value = setting[1].strip("\n")
+                match settings_key:
+                    case "gid" | "uid" | "size":
+                        settings_dict[settings_key] = int(setting_value)
+                    case "time":
+                        settings_dict[settings_key] = float(setting_value)
+                    case "type":
+                        # NOTE: do not overload type()
+                        settings_dict["type_"] = setting_value
+                    case "mode":
+                        # NOTE: ensure that file modes are zero-filled and of length 4
+                        settings_dict[settings_key] = setting_value.zfill(4)
+                    case "md5digest" | "sha1digest" | "sha256digest" | "sha384digest" | "sha512digest":
+                        # NOTE: only use the name of the digest as key to be more flexible in reusing the model
+                        settings_dict[settings_key.replace("digest", "")] = setting_value
+                    case _:
+                        settings_dict[settings_key] = setting_value
+
+        base_settings: dict[str, float | int | str] = {}
+        entries: list[MTreeEntry] = []
 
         for line in data:
             if line.startswith("/set"):
-                settings = [assignment.split("=") for assignment in line.split(" ")[1:]]
-                for setting in settings:
-                    setting_value = setting[1].strip("\n")
-                    match setting[0]:
-                        case "gid" | "uid":
-                            base_settings[setting[0]] = int(setting_value)
-                        case "type":
-                            base_settings["type_"] = setting_value
-                        case _:
-                            base_settings[setting[0]] = setting_value
+                settings_list = [assignment.split("=") for assignment in line.split(" ")[1:]]
+                sanitize_mtree_pairs(settings_list=settings_list, settings_dict=base_settings)
 
             elif line.startswith("."):
-                file_settings: Dict[str, Union[float, int, str]] = {}
+                file_settings: dict[str, float | int | str] = {}
                 file_settings["name"] = line.split()[0][1:]
 
                 # provide a list of all settings in an entry line (skip empty assigments due to multiple whitespace)
                 settings_list = [assignment.split("=") for assignment in line.split()[1:] if assignment]
-                for setting in settings_list:
-                    setting_value = setting[1].strip("\n")
-                    match setting[0]:
-                        case "gid" | "uid" | "size":
-                            file_settings[setting[0]] = int(setting_value)
-                        case "time":
-                            file_settings[setting[0]] = float(setting_value)
-                        case "type":
-                            # NOTE: do not overload type()
-                            file_settings["type_"] = setting_value
-                        case "md5digest" | "sha1digest" | "sha256digest" | "sha384digest" | "sha512digest":
-                            # NOTE: only use the name of the digest as key to be more flexible in reusing the model
-                            file_settings[setting[0].replace("digest", "")] = setting_value
-                        case _:
-                            file_settings[setting[0]] = setting_value
+                sanitize_mtree_pairs(settings_list=settings_list, settings_dict=file_settings)
 
                 try:
                     entries.append(
                         MTreeEntryV1(
                             gid=file_settings.get("gid") or base_settings.get("gid"),
                             link=file_settings.get("link"),
-                            md5=file_settings.get("md5"),
+                            md5=file_settings.get("md5") or base_settings.get("md5"),
                             mode=file_settings.get("mode") or base_settings.get("mode"),
                             name=file_settings.get("name"),
-                            sha256=file_settings.get("sha256"),
-                            size=file_settings.get("size"),
-                            time=file_settings.get("time"),
+                            sha256=file_settings.get("sha256") or base_settings.get("sha256"),
+                            size=file_settings.get("size") or base_settings.get("size"),  # type: ignore[arg-type]
+                            time=file_settings.get("time") or base_settings.get("time"),  # type: ignore[arg-type]
                             type_=file_settings.get("type_") or base_settings.get("type_"),
                             uid=file_settings.get("uid") or base_settings.get("uid"),
                         )
@@ -386,8 +395,8 @@ class MTree(BaseModel):
 
         return MTree(entries=entries)
 
-    def get_paths(self, show_all: bool = False) -> List[Path]:
-        """Return the list of Paths described by the entries of the MTree
+    def get_paths(self, show_all: bool = False) -> list[Path]:
+        """Return the list of Paths described by the entries of the MTree.
 
         Parameters
         ----------
@@ -396,11 +405,10 @@ class MTree(BaseModel):
 
         Returns
         -------
-        List[Path]
+        list[Path]
             A list of Paths
         """
-
-        path_list: List[Path] = []
+        path_list: list[Path] = []
 
         for entry in self.entries:
             path = entry.get_file_path()
@@ -412,8 +420,8 @@ class MTree(BaseModel):
         return path_list
 
 
-def export_schemas(output: Union[Path, str]) -> None:
-    """Export the JSON schema of selected pydantic models to an output directory
+def export_schemas(output: Path | str) -> None:
+    """Export the JSON schema of selected pydantic models to an output directory.
 
     Parameters
     ----------
@@ -425,7 +433,6 @@ def export_schemas(output: Union[Path, str]) -> None:
     RuntimeError
         If output is not an existing directory
     """
-
     classes = [MTreeEntryV1]
 
     if isinstance(output, str):
@@ -435,5 +442,10 @@ def export_schemas(output: Union[Path, str]) -> None:
         raise RuntimeError(f"The output directory {output} must exist!")
 
     for class_ in classes:
-        with open(output / f"{class_.__name__}.json", "w") as f:
-            print(class_.schema_json(indent=2), file=f)
+        with open(output / f"{class_.__name__}.json", "wb") as f:
+            f.write(
+                dumps(
+                    class_.model_json_schema(),
+                    option=OPT_INDENT_2 | OPT_APPEND_NEWLINE | OPT_SORT_KEYS,
+                )
+            )
