@@ -1,21 +1,19 @@
+"""Handling of package files and their contents."""
 from __future__ import annotations
 
 from base64 import b64encode
 from hashlib import md5, sha256
 from logging import debug, info
 from pathlib import Path
-from typing import Any, Dict, Optional, Union
+from typing import Any
 
-from pydantic import BaseModel
+from orjson import OPT_APPEND_NEWLINE, OPT_INDENT_2, OPT_SORT_KEYS, dumps
+from pydantic import BaseModel, SerializeAsAny
 
 from repod.common.models import CSize, FileName, Md5Sum, PgpSig, Sha256Sum
 from repod.errors import RepoManagementFileError
 from repod.files.buildinfo import BuildInfo
-from repod.files.common import (  # noqa: F401
-    extract_file_from_tarfile,
-    names_in_tarfile,
-    open_tarfile,
-)
+from repod.files.common import extract_file_from_tarfile, names_in_tarfile, open_tarfile
 from repod.files.mtree import MTree
 from repod.files.pkginfo import PkgInfo
 
@@ -27,21 +25,21 @@ PACKAGE_VERSIONS = {
 
 
 class Package(BaseModel):
-    """Package representation
+    """Package representation.
 
     This is a template class and (apart from its class methods) should not be used directly. Instead instatiate one of
     the classes derived from it.
     """
 
     @classmethod
-    async def from_file(cls, package: Path, signature: Optional[Path] = None) -> Package:
-        """Create a Package from a package file and an optional signature
+    async def from_file(cls, package: Path, signature: Path | None = None) -> Package:
+        """Create a Package from a package file and an optional signature.
 
         Parameters
         ----------
         package: Path
             The path to a package file
-        signature: Optional[Path]
+        signature: Path | None
             The optional path to a signature file for package
 
         Raises
@@ -55,11 +53,10 @@ class Package(BaseModel):
         Package
             A Package representing the metadata contained in package and the optional signature file
         """
-
         package_version = 0
         package_md5sum = ""
         package_sha256sum = ""
-        pgpsig: Optional[str] = None
+        pgpsig: str | None = None
 
         if signature:
             debug(f"Opening signature file {signature} for reading...")
@@ -79,7 +76,8 @@ class Package(BaseModel):
         debug(f"Creating checksums for package {package}...")
         with open(package, "rb") as package_file:
             package_bytes = package_file.read()
-            package_md5sum = md5(package_bytes).hexdigest()
+            # NOTE: MD5 sums are still part of the PackageV1 API
+            package_md5sum = md5(package_bytes).hexdigest()  # nosec: B324
             package_sha256sum = sha256(package_bytes).hexdigest()
 
         debug(f"Opening package file {package} for reading...")
@@ -127,19 +125,18 @@ class Package(BaseModel):
                         f"The provided file {package} does not match any known package versions!"
                     )
 
-    def top_level_dict(self) -> Dict[str, Any]:
-        """Flatten the keys and values tracked by Package (one level deep) and return them in a dict
+    def top_level_dict(self) -> dict[str, Any]:
+        """Flatten the keys and values tracked by Package (one level deep) and return them in a dict.
 
         NOTE: Duplicate entries are merged!
 
         Returns
         -------
-        Dict[str, Any]
+        dict[str, Any]
             A flattened dict representation of Package
         """
-
-        top_level: Dict[str, Any] = {}
-        for key, value in self.dict().items():
+        top_level: dict[str, Any] = {}
+        for key, value in self.model_dump(mode="python").items():
             if isinstance(value, dict):
                 top_level.update(value)
             else:
@@ -149,7 +146,7 @@ class Package(BaseModel):
 
 
 class PackageV1(CSize, FileName, Md5Sum, Package, PgpSig, Sha256Sum):
-    """Package representation version 1
+    """Package representation version 1.
 
     Attributes
     ----------
@@ -163,7 +160,7 @@ class PackageV1(CSize, FileName, Md5Sum, Package, PgpSig, Sha256Sum):
         An MD5 checksum for the package
     mtree: MTree
         An .MTREE file representation
-    pgpsig: Optional[str]
+    pgpsig: str | None
         An optional PGP signature (in base64 representation) for the package
     pkginfo: PkgInfo
         A .PKGINFO file representation
@@ -171,13 +168,13 @@ class PackageV1(CSize, FileName, Md5Sum, Package, PgpSig, Sha256Sum):
         A SHA256 checksum for the package
     """
 
-    buildinfo: BuildInfo
-    mtree: MTree
-    pkginfo: PkgInfo
+    buildinfo: SerializeAsAny[BuildInfo]
+    mtree: SerializeAsAny[MTree]
+    pkginfo: SerializeAsAny[PkgInfo]
 
 
-def export_schemas(output: Union[Path, str]) -> None:
-    """Export the JSON schema of selected pydantic models to an output directory
+def export_schemas(output: Path | str) -> None:
+    """Export the JSON schema of selected pydantic models to an output directory.
 
     Parameters
     ----------
@@ -189,7 +186,6 @@ def export_schemas(output: Union[Path, str]) -> None:
     RuntimeError
         If output is not an existing directory
     """
-
     classes = [PackageV1]
 
     if isinstance(output, str):
@@ -199,5 +195,10 @@ def export_schemas(output: Union[Path, str]) -> None:
         raise RuntimeError(f"The output directory {output} must exist!")
 
     for class_ in classes:
-        with open(output / f"{class_.__name__}.json", "w") as f:
-            print(class_.schema_json(indent=2), file=f)
+        with open(output / f"{class_.__name__}.json", "wb") as f:
+            f.write(
+                dumps(
+                    class_.model_json_schema(),
+                    option=OPT_INDENT_2 | OPT_APPEND_NEWLINE | OPT_SORT_KEYS,
+                )
+            )
