@@ -1,5 +1,5 @@
+"""Handling of management and package repository files."""
 from pathlib import Path
-from typing import Union
 
 from repod.repo.management import (  # noqa: F401
     Files,
@@ -24,14 +24,13 @@ from repod.repo.package import (  # noqa: F401
 )
 
 
-def export_schemas(output: Union[Path, str]) -> None:
-    """Export the JSON schema files of the repod.repo package to a directory
+def export_schemas(output: Path | str) -> None:
+    """Export the JSON schema files of the repod.repo package to a directory.
 
     Parameters
     ----------
     output: Path
         A directory to write the JSON schema files to
     """
-
     management_export_schemas(output=output)
     package_export_schemas(output=output)
