@@ -1,12 +1,20 @@
+"""Reading and writing of OutputPackageBase and OutputPackage."""
 from __future__ import annotations
 
 from logging import debug
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Set, Tuple, Union
+from typing import Any
 
 from aiofiles import open as async_open
-from orjson import JSONDecodeError, loads
-from pydantic import BaseModel, ValidationError
+from orjson import (
+    OPT_APPEND_NEWLINE,
+    OPT_INDENT_2,
+    OPT_SORT_KEYS,
+    JSONDecodeError,
+    dumps,
+    loads,
+)
+from pydantic import BaseModel, SerializeAsAny, ValidationError
 
 from repod import errors
 from repod.common.enums import (
@@ -37,7 +45,9 @@ from repod.common.models import (
     Provides,
     Replaces,
     SchemaVersionV1,
+    SchemaVersionV2,
     Sha256Sum,
+    SourceUrl,
     Url,
     Version,
 )
@@ -50,8 +60,6 @@ from repod.files.buildinfo import (
     BuildInfoV2,
     BuildTool,
     BuildToolVer,
-    FormatV1,
-    FormatV2,
     Installed,
     Options,
     PkgBuildSha256Sum,
@@ -65,7 +73,7 @@ from repod.repo.package.syncdb import (
     PackageDescV2,
 )
 
-OUTPUT_PACKAGE_VERSIONS: Dict[int, Dict[str, Set[str]]] = {
+OUTPUT_PACKAGE_VERSIONS: dict[int, dict[str, set[str]]] = {
     1: {
         "required": {
             "arch",
@@ -94,7 +102,7 @@ OUTPUT_PACKAGE_VERSIONS: Dict[int, Dict[str, Set[str]]] = {
         },
     },
 }
-OUTPUT_PACKAGE_BASE_VERSIONS: Dict[int, Dict[str, Union[int, Set[str]]]] = {
+OUTPUT_PACKAGE_BASE_VERSIONS: dict[int, dict[str, int | set[str]]] = {
     1: {
         "required": {
             "base",
@@ -109,7 +117,7 @@ DEFAULT_OUTPUT_PACKAGE_BASE_VERSION = 1
 
 
 class OutputBuildInfo(BaseModel):
-    """A class tracking BuildInfo information of packages that are added to instances of OutputPackageBase
+    """A class tracking BuildInfo information of packages that are added to instances of OutputPackageBase.
 
     This class is a base template class and should not be used directly.
     Instead, instantiate one of its versioned child classes using the `from_buildinfo()` classmethod.
@@ -117,7 +125,7 @@ class OutputBuildInfo(BaseModel):
 
     @classmethod
     def from_buildinfo(cls, buildinfo: BuildInfo) -> OutputBuildInfo:
-        """Create and return an instance of one of OutputBuildInfo's versioned child classes
+        """Create and return an instance of one of OutputBuildInfo's versioned child classes.
 
         Parameters
         ----------
@@ -135,15 +143,14 @@ class OutputBuildInfo(BaseModel):
         OutputBuildInfo
             One of OutputBuildInfo's versioned child classes (e.g. OutputBuildInfoV1 or OutputBuildInfoV2)
         """
-
         if isinstance(buildinfo, BuildInfoV1):
             return OutputBuildInfoV1(
                 builddir=buildinfo.builddir,
                 buildenv=buildinfo.buildenv,
-                format_=buildinfo.format_,
                 installed=buildinfo.installed,
                 options=buildinfo.options,
                 pkgbuild_sha256sum=buildinfo.pkgbuild_sha256sum,
+                schema_version=buildinfo.schema_version,
             )
         elif isinstance(buildinfo, BuildInfoV2):
             return OutputBuildInfoV2(
@@ -151,10 +158,10 @@ class OutputBuildInfo(BaseModel):
                 buildenv=buildinfo.buildenv,
                 buildtool=buildinfo.buildtool,
                 buildtoolver=buildinfo.buildtoolver,
-                format_=buildinfo.format_,
                 installed=buildinfo.installed,
                 options=buildinfo.options,
                 pkgbuild_sha256sum=buildinfo.pkgbuild_sha256sum,
+                schema_version=buildinfo.schema_version,
                 startdir=buildinfo.startdir,
             )
         else:
@@ -167,13 +174,13 @@ class OutputBuildInfo(BaseModel):
 class OutputBuildInfoV1(
     BuildDir,
     BuildEnv,
-    FormatV1,
     Installed,
     Options,
     OutputBuildInfo,
     PkgBuildSha256Sum,
+    SchemaVersionV1,
 ):
-    """OutputBuildInfo version 1
+    """OutputBuildInfo version 1.
 
     Attributes which are already covered by OutputPackageBase are ommitted.
     Instances of this class relate to OutputBuildInfo the same way as BuildinfoV1 relates to BuildInfo.
@@ -182,17 +189,17 @@ class OutputBuildInfoV1(
     ----------
     builddir: str
         A string representing an absolute directory
-    buildenv: List[str]
+    buildenv: list[str]
         A list of strings as described by makepkg.conf's BUILDENV option
-    format_: int
-        An integer describing a BuildInfo format version
-    installed: List[str]
+    installed: list[str]
         A list of strings representing <package_name>-<epoch><version>-<pkgrel>-<architecture> of packages installed
         during the creation of a package
-    options: List[str]
+    options: list[str]
         A list of strings representing makepkg.conf OPTIONS used during the creation of a package
     pkgbuild_sha256sum: str
         A string representing a SHA-256 checksum for a PKGBUILD of a package
+    schema_version: int
+        An integer describing a BuildInfo format version
     """
 
     pass
@@ -203,14 +210,14 @@ class OutputBuildInfoV2(
     BuildEnv,
     BuildTool,
     BuildToolVer,
-    FormatV2,
     Installed,
     Options,
     OutputBuildInfo,
     PkgBuildSha256Sum,
+    SchemaVersionV2,
     StartDir,
 ):
-    """OutputBuildInfo version 2
+    """OutputBuildInfo version 2.
 
     Attributes which are already covered by OutputPackageBase are ommitted.
     Instances of this class relate to OutputBuildInfo the same way as BuildinfoV2 relates to BuildInfo.
@@ -219,21 +226,21 @@ class OutputBuildInfoV2(
     ----------
     builddir: str
         A string representing an absolute directory
-    buildenv: List[str]
+    buildenv: list[str]
         A list of strings as described by makepkg.conf's BUILDENV option
     buildtool: str
         The package name of the build tool used to create a package
     buildtoolver: str
         The version of the build tool used to create a package
-    format_: int
-        An integer describing a BuildInfo format version
-    installed: List[str]
+    installed: list[str]
         A list of strings representing <package_name>-<epoch><version>-<pkgrel>-<architecture> of packages installed
         during the creation of a package
-    options: List[str]
+    options: list[str]
         A list of strings representing makepkg.conf OPTIONS used during the creation of a package
     pkgbuild_sha256sum: str
         A string representing a SHA-256 checksum for a PKGBUILD of a package
+    schema_version: int
+        An integer describing a BuildInfo format version
     startdir: str
         A string representing the absolute startdir directory of a package
     """
@@ -242,20 +249,20 @@ class OutputBuildInfoV2(
 
 
 class OutputPackage(BaseModel):
-    """A template class to describe all required attributes that define a package in the context of an output file
+    """A template class to describe all required attributes that define a package in the context of an output file.
 
     This class should not be instantiated directly. Use one of its subclasses instead!
     """
 
     @classmethod
     def from_package(cls, package: package.Package) -> OutputPackage:
-        """Create an OutputPackage from a Package
+        """Create an OutputPackage from a Package.
 
         Parameters
         ----------
         package: Path
             The path to a package file
-        signature: Optional[Path]
+        signature: Path | None
             The optional path to a signature file for package
 
         Returns
@@ -263,7 +270,6 @@ class OutputPackage(BaseModel):
         OutputPackage
             An instance of one of OutputPackage's child classes
         """
-
         outputpackage_version = 0
         data = package.top_level_dict()
         keys = set(data.keys())
@@ -284,7 +290,7 @@ class OutputPackage(BaseModel):
                     backup=package.pkginfo.backup,  # type: ignore[attr-defined]
                     builddate=package.pkginfo.builddate,  # type: ignore[attr-defined]
                     checkdepends=package.pkginfo.checkdepends,  # type: ignore[attr-defined]
-                    conflicts=package.pkginfo.checkdepends,  # type: ignore[attr-defined]
+                    conflicts=package.pkginfo.conflicts,  # type: ignore[attr-defined]
                     csize=package.csize,  # type: ignore[attr-defined]
                     depends=package.pkginfo.depends,  # type: ignore[attr-defined]
                     desc=package.pkginfo.desc,  # type: ignore[attr-defined]
@@ -340,29 +346,29 @@ class OutputPackageV1(
     Sha256Sum,
     Url,
 ):
-    """A model describing all required attributes that define a package in the context of an output file (version 1)
+    """A model describing all required attributes that define a package in the context of an output file (version 1).
 
     Attributes
     ----------
     arch: str
         The attribute can be used to describe the (required) data below an %ARCH% identifier in a 'desc' file, which
         identifies a package's architecture
-    backup: Optional[List[str]]
+    backup: list[str] | None
         The attribute can be used to describe the (optional) data below a %BACKUP% identifier in a 'desc' file, which
         identifies which file(s) of a package pacman will create backups for
     builddate: int
         The attribute can be used to describe the (required) data below a %BUILDDATE% identifier in a 'desc' file,
         which identifies a package's build date (represented in seconds since the epoch)
-    checkdepends: Optional[List[str]]
+    checkdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %CHECKDEPENDS% identifier in a 'desc' file,
         which identifies a package's checkdepends
-    conflicts: Optional[List[str]]
+    conflicts: list[str] | None
         The attribute can be used to describe the (optional) data below a %CONFLICTS% identifier in a 'desc' file, which
         identifies what other package(s) a package conflicts with
     csize: int
         The attribute can be used to describe the (required) data below a %CSIZE% identifier in a 'desc' file, which
         identifies a package's size
-    depends: Optional[List[str]]
+    depends: list[str] | None
         The attribute can be used to describe the (optional) data below a %DEPENDS% identifier in a 'desc' file, which
         identifies what other package(s) a package depends on
     desc: str
@@ -371,16 +377,16 @@ class OutputPackageV1(
     filename: str
         The attribute can be used to describe the (required) data below a %FILENAME% identifier in a 'desc' file, which
         identifies a package's file name
-    files: Optional[List[str]]
+    files: list[str] | None
         The attribute can be used to describe the (optional) data below a %FILES% identifier in a 'files' file, which
         identifies which file(s) belong to a package
-    groups: Optional[List[str]]
+    groups: list[str] | None
         The attribute can be used to describe the (optional) data below a %GROUPS% identifier in a 'desc' file, which
         identifies a package's groups
     isize: int
         The attribute can be used to describe the (required) data below an %ISIZE% identifier in a 'desc' file, which
         identifies a package's installed size
-    license: List[str]
+    license: list[str]
         The attribute can be used to describe the (required) data below a %LICENSE% identifier in a 'desc' file, which
         identifies a package's license(s)
     md5sum: str
@@ -389,16 +395,16 @@ class OutputPackageV1(
     name: str
         The attribute can be used to describe the (required) data below a %NAME% identifier in a 'desc' file, which
         identifies a package's name
-    optdepends: Optional[List[str]]
+    optdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %OPTDEPENDS% identifier in a 'desc' file,
         which identifies what other package(s) a package optionally depends on
     pgpsig: str
         The attribute can be used to describe the (required) data below a %PGPSIG% identifier in a 'desc' file, which
         identifies a package's PGP signature
-    provides: Optional[List[str]]
+    provides: list[str] | None
         The attribute can be used to describe the (optional) data below a %PROVIDES% identifier in a 'desc' file, which
         identifies what other package(s) a package provides
-    replaces: Optional[List[str]]
+    replaces: list[str] | None
         The attribute can be used to describe the (optional) data below a %REPLACES% identifier in a 'desc' file, which
         identifies what other package(s) a package replaces
     schema_version: PositiveInt
@@ -411,11 +417,11 @@ class OutputPackageV1(
         identifies a package's URL
     """
 
-    files: Optional[Files] = None
+    files: SerializeAsAny[Files | None] = None
 
 
 class OutputPackageBase(BaseModel):
-    """A template class with helper methods to create instances of one of its (versioned) subclasses
+    """A template class with helper methods to create instances of one of its (versioned) subclasses.
 
     This class should not be instantiated directly, as it only provides generic instance methods for its subclasses.
 
@@ -423,8 +429,8 @@ class OutputPackageBase(BaseModel):
     """
 
     @classmethod
-    def from_dict(cls, data: Dict[str, Union[Any, List[Any]]]) -> OutputPackageBase:
-        """Create an instance of one of OutputPackageBase's subclasses from a dict
+    def from_dict(cls, data: dict[str, Any | list[Any]]) -> OutputPackageBase:
+        """Create an instance of one of OutputPackageBase's subclasses from a dict.
 
         This method expects data derived from reading a pkgbase JSON file from the management repository.
         By default this method will prefer to create an instances of OutputPackageBase's subclasses as identified by
@@ -432,7 +438,7 @@ class OutputPackageBase(BaseModel):
 
         Parameters
         ----------
-        data: Dict[str, Union[Any, List[Any]]]
+        data: dict[str, Any | list[Any]]
             A dict containing data required to instantiate a subclass of OutputPackageBase
 
         Raises
@@ -448,15 +454,15 @@ class OutputPackageBase(BaseModel):
             An instance of one of the subclasses of OutputPackageBase
         """
 
-        def default_output_package_from_dict(version: int, package: Dict[str, Any]) -> OutputPackage:
-            """Create the default OutputPackage subclass for a OutputPackageBase from a dict
+        def default_output_package_from_dict(version: int, package: dict[str, Any]) -> OutputPackage:
+            """Create the default OutputPackage subclass for a OutputPackageBase from a dict.
 
             Parameters
             ----------
             version: int
                 The schema_version of the OutputPackageBase subclass for which to create an instance of a subclass of
                 OutputPackage
-            package: Dict[str, Any]
+            package: dict[str, Any]
                 A dict describing the attributes of an OutputPackage instance
 
             Returns
@@ -464,7 +470,6 @@ class OutputPackageBase(BaseModel):
             OutputPackage
                 A subclass of OutputPackage that is the default for the given OutputPackageBase schema_version
             """
-
             outputpackage_version = OutputPackageVersionEnum.DEFAULT.value
             files_version = FilesVersionEnum.DEFAULT.value
 
@@ -474,9 +479,9 @@ class OutputPackageBase(BaseModel):
                     if files:
                         if files.get("schema_version"):
                             del files["schema_version"]
-                        package["files"] = FilesV1(**files)
+                        package["files"] = FilesV1.model_validate(files)
 
-                    return OutputPackageV1(**package)
+                    return OutputPackageV1.model_validate(package)
                 # NOTE: the catch all can never be reached but is here to satisfy our tooling
                 case _:  # pragma: no cover
                     raise RuntimeError(
@@ -503,7 +508,7 @@ class OutputPackageBase(BaseModel):
                         for package in data.get("packages")  # type: ignore[union-attr]
                     ]
                 try:
-                    return OutputPackageBaseV1(**data)
+                    return OutputPackageBaseV1.model_validate(data)
                 except ValidationError as e:
                     raise errors.RepoManagementValidationError(
                         "A validation error occured while attempting to instantiate an OutputPackageBaseV1 using data:"
@@ -517,7 +522,7 @@ class OutputPackageBase(BaseModel):
 
     @classmethod
     async def from_file(cls, path: Path) -> OutputPackageBase:
-        """Initialize an OutputPackageBase from a JSON file
+        """Initialize an OutputPackageBase from a JSON file.
 
         Parameters
         ----------
@@ -534,7 +539,6 @@ class OutputPackageBase(BaseModel):
         OutputPackageBase
             An instance of OutputPackageBase based on path
         """
-
         async with async_open(path, "r") as input_file:
             try:
                 return OutputPackageBase.from_dict(data=loads(await input_file.read()))
@@ -542,12 +546,12 @@ class OutputPackageBase(BaseModel):
                 raise errors.RepoManagementFileError(f"The JSON file '{path}' could not be decoded!\n{e}")
 
     @classmethod
-    def from_package(cls, packages: List[package.Package]) -> OutputPackageBase:
-        """Create an OutputPackageBase from a list of Packages of the same pkgbase, pkgtype and version
+    def from_package(cls, packages: list[package.Package]) -> OutputPackageBase:
+        """Create an OutputPackageBase from a list of Packages of the same pkgbase, pkgtype and version.
 
         Parameters
         ----------
-        packages: List[Package]
+        packages: list[Package]
             A list of Packages to create an OutputPackageBase for
 
         Raises
@@ -564,7 +568,6 @@ class OutputPackageBase(BaseModel):
         OutputPackageBase
             An OutputPackageBase that is based on packages
         """
-
         if len(packages) == 0:
             raise ValueError("At least one Package needs to be provided to create an OutputPackageBase.")
 
@@ -591,16 +594,16 @@ class OutputPackageBase(BaseModel):
                 f"provided: {', '.join(versions)}"
             )
 
-        all_xdata: List[List[Dict[str, Any]]] = [
+        all_xdata: list[list[dict[str, Any]]] = [
             pkg.top_level_dict().get("xdata")  # type: ignore[misc]
             for pkg in packages
             if pkg.top_level_dict().get("xdata")
         ]
         debug(f"all xdata: {all_xdata}")
-        xdata: List[Dict[str, str]] = []
+        xdata: list[dict[str, str]] = []
         xdata = [item for sublist in all_xdata for item in sublist]
         debug(f"collected xdata: {xdata}")
-        pkgtypes: List[str] = []
+        pkgtypes: list[str] = []
         pkgtypes = [data.get("pkgtype").value for data in xdata if data.get("pkgtype")]  # type: ignore[union-attr]
         debug(f"collected pkgtypes: {pkgtypes}")
 
@@ -648,15 +651,15 @@ class OutputPackageBase(BaseModel):
                     f"Unable to find matching version for Package keys: {keys}"
                 )
 
-    def add_packages(self, packages: List[OutputPackage]) -> None:
-        """Add packages to an instance of one of OutputPackageBase's subclasses
+    def add_packages(self, packages: list[OutputPackage]) -> None:
+        """Add packages to an instance of one of OutputPackageBase's subclasses.
 
         NOTE: This method only operates successfully if the instance of the class using it actually defines the
         `packages` field! The OutputPackageBase class does not do that!
 
         Parameters
         ----------
-        packages: List[OutputPackage]
+        packages: list[OutputPackage]
             A list of OutputPackage instances to add
 
         Raises
@@ -664,16 +667,15 @@ class OutputPackageBase(BaseModel):
         RuntimeError
             If called on the OutputPackageBase template class
         """
-
         if hasattr(self, "packages"):
             # TODO: only add OutputPackage subclasses that are compatible with the OutputPackageBase version, else
             # attempt conversion
-            self.packages += packages  # type: ignore[attr-defined]
+            self.packages += packages
         else:
             raise RuntimeError("It is not possible to add packages to the template class OutputPackageBase!")
 
     def get_version(self) -> str:
-        """Get version of an instance of one of OutputPackage's subclasses
+        """Get version of an instance of one of OutputPackage's subclasses.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the `get_version`
         field! The OutputPackageBase class does not do that!
@@ -688,9 +690,8 @@ class OutputPackageBase(BaseModel):
         str
             The version string of the OutputPackageBase
         """
-
         if hasattr(self, "version"):
-            return str(self.version)  # type: ignore[attr-defined]
+            return str(self.version)
         else:
             raise RuntimeError(
                 "It is not possible to return the version attribute of the template class OutputPackageBase!"
@@ -700,8 +701,8 @@ class OutputPackageBase(BaseModel):
         self,
         packagedesc_version: PackageDescVersionEnum = PackageDescVersionEnum.DEFAULT,
         files_version: FilesVersionEnum = FilesVersionEnum.DEFAULT,
-    ) -> List[Tuple[PackageDesc, Files]]:
-        """Return the list of packages as tuples of PackageDesc and Files models
+    ) -> list[tuple[PackageDesc, Files]]:
+        """Return the list of packages as tuples of PackageDesc and Files models.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the required fields!
         The OutputPackageBase class does not do that!
@@ -722,19 +723,16 @@ class OutputPackageBase(BaseModel):
 
         Returns
         -------
-        List[Tuple[PackageDesc, Files]]
+        list[tuple[PackageDesc, Files]]
             A list of tuples with one PackageDesc and one Files each
         """
-
         if not hasattr(self, "schema_version"):
             raise RuntimeError(
                 "Packages and their files can not be retrieved from the templatae class OutputPackageBase!"
             )
 
-        if self.schema_version not in OUTPUT_PACKAGE_BASE_VERSIONS.keys():  # type: ignore[attr-defined]
-            raise RuntimeError(
-                f"OutputPackageBase has invalid schema_version {self.schema_version}!"  # type: ignore[attr-defined]
-            )
+        if self.schema_version not in OUTPUT_PACKAGE_BASE_VERSIONS.keys():
+            raise RuntimeError(f"OutputPackageBase has invalid schema_version {self.schema_version}!")
 
         match (packagedesc_version, files_version):
             case (PackageDescVersionEnum.ONE, FilesVersionEnum.DEFAULT):
@@ -816,40 +814,42 @@ class OutputPackageBaseV1(
     MakeDepends,
     Packager,
     SchemaVersionV1,
+    SourceUrl,
     Version,
 ):
-    """A model describing all required attributes for an output file, that describes a list of packages based upon a
-    pkgbase (version 1)
+    """A model for an output file format, that describes a list of packages based upon a pkgbase (version 1).
 
     Attributes
     ----------
     base: str
         The attribute can be used to describe the (required) data below a %BASE% identifier in a 'desc' file, which
         identifies a package's pkgbase
-    buildinfo: Optional[OutputBuildInfo]
+    buildinfo: OutputBuildInfo | None
         An optional OutputBuildInfo, which describes the build circumstances of the OutputPackageBase. The data is not
         covered in a repository sync database and therefore optional.
-    makedepends: Optional[List[str]]
+    makedepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %MAKEDEPENDS% identifier in a 'desc' file,
         which identifies a package's makedepends
     packager: str
         The attribute can be used to describe the (required) data below a %PACKAGER% identifier in a 'desc' file, which
         identifies a package's packager
-    packages: List[OutputPackage]
+    packages: list[OutputPackage]
         A list of OutputPackage instances that belong to the pkgbase identified by base
     schema_version: PositiveInt
         A positive integer - 1 - identifying the schema version of the object
+    source_url: HttpUrl | None
+        An optional url that points at sources (defaults to None)
     version: str
         The attribute can be used to describe the (required) data below a %VERSION% identifier in a 'desc' file, which
         identifies a package's version (this is the accumulation of epoch, pkgver and pkgrel)
     """
 
-    buildinfo: Optional[OutputBuildInfo]
-    packages: List[OutputPackage]
+    buildinfo: SerializeAsAny[OutputBuildInfo | None] = None
+    packages: SerializeAsAny[list[OutputPackage]]
 
 
-def export_schemas(output: Union[Path, str]) -> None:
-    """Export the JSON schema of selected pydantic models to an output directory
+def export_schemas(output: Path | str) -> None:
+    """Export the JSON schema of selected pydantic models to an output directory.
 
     Parameters
     ----------
@@ -861,7 +861,6 @@ def export_schemas(output: Union[Path, str]) -> None:
     RuntimeError
         If output is not an existing directory
     """
-
     classes = [OutputBuildInfoV1, OutputBuildInfoV2, OutputPackageV1, OutputPackageBaseV1]
 
     if isinstance(output, str):
@@ -871,5 +870,10 @@ def export_schemas(output: Union[Path, str]) -> None:
         raise RuntimeError(f"The output directory {output} must exist!")
 
     for class_ in classes:
-        with open(output / f"{class_.__name__}.json", "w") as f:
-            print(class_.schema_json(indent=2), file=f)
+        with open(output / f"{class_.__name__}.json", "wb") as f:
+            f.write(
+                dumps(
+                    class_.model_json_schema(),  # type: ignore[attr-defined]
+                    option=OPT_INDENT_2 | OPT_APPEND_NEWLINE | OPT_SORT_KEYS,
+                )
+            )
