@@ -1,3 +1,4 @@
+"""Sync database handling."""
 from __future__ import annotations
 
 import io
@@ -7,9 +8,9 @@ from logging import debug, warning
 from pathlib import Path
 from tarfile import DIRTYPE, TarFile, TarInfo
 from time import time
-from typing import Dict, List, Optional, Set, Tuple, Union
 
 from jinja2 import Environment, PackageLoader, TemplateNotFound
+from orjson import OPT_APPEND_NEWLINE, OPT_INDENT_2, OPT_SORT_KEYS, dumps
 from pydantic import BaseModel, ValidationError
 
 from repod.common.enums import (
@@ -58,7 +59,7 @@ DB_USER = "root"
 DB_GROUP = "root"
 DB_FILE_MODE = "0644"
 DB_DIR_MODE = "0755"
-DESC_JSON: Dict[str, Tuple[str, FieldTypeEnum]] = {
+DESC_JSON: dict[str, tuple[str, FieldTypeEnum]] = {
     "%BASE%": ("base", FieldTypeEnum.STRING),
     "%VERSION%": ("version", FieldTypeEnum.STRING),
     "%MAKEDEPENDS%": ("makedepends", FieldTypeEnum.STRING_LIST),
@@ -84,8 +85,8 @@ DESC_JSON: Dict[str, Tuple[str, FieldTypeEnum]] = {
     "%OPTDEPENDS%": ("optdepends", FieldTypeEnum.STRING_LIST),
     "%BACKUP%": ("backup", FieldTypeEnum.STRING_LIST),
 }
-FILES_JSON: Dict[str, Tuple[str, FieldTypeEnum]] = {"%FILES%": ("files", FieldTypeEnum.STRING_LIST)}
-FILES_VERSIONS: Dict[int, Dict[str, Set[str]]] = {
+FILES_JSON: dict[str, tuple[str, FieldTypeEnum]] = {"%FILES%": ("files", FieldTypeEnum.STRING_LIST)}
+FILES_VERSIONS: dict[int, dict[str, set[str]]] = {
     1: {
         "required": {
             "files",
@@ -95,7 +96,7 @@ FILES_VERSIONS: Dict[int, Dict[str, Set[str]]] = {
 }
 DEFAULT_FILES_VERSION = 1
 DEFAULT_PACKAGE_DESC_VERSION = 1
-PACKAGE_DESC_VERSIONS: Dict[int, Dict[str, Union[Set[str], int]]] = {
+PACKAGE_DESC_VERSIONS: dict[int, dict[str, set[str] | int]] = {
     1: {
         "required": {
             "arch",
@@ -181,7 +182,7 @@ class RepoDbMemberTypeEnum(IntEnum):
 
 
 class RepoDbTypeEnum(IntEnum):
-    """An IntEnum to distinguish types of binary repository database files
+    """An IntEnum to distinguish types of binary repository database files.
 
     Attributes
     ----------
@@ -195,20 +196,19 @@ class RepoDbTypeEnum(IntEnum):
     FILES = 2
 
 
-def get_desc_json_keys() -> Set[str]:
-    """Get the keys of repod.models.repo.DESC_JSON
+def get_desc_json_keys() -> set[str]:
+    """Get the keys of repod.models.repo.DESC_JSON.
 
     Returns
     -------
-    Set[str]
+    set[str]
         A set of strings representing the keys of repod.models.repo.DESC_JSON
     """
-
     return set(DESC_JSON.keys())
 
 
 def get_desc_json_name(key: str) -> str:
-    """Get the JSON name of a given key from the definition in repod.models.repo.DESC_JSON
+    """Get the JSON name of a given key from the definition in repod.models.repo.DESC_JSON.
 
     Parameters
     ----------
@@ -225,7 +225,6 @@ def get_desc_json_name(key: str) -> str:
     str
         The JSON name of a given 'desc' file identifier provided by key
     """
-
     try:
         return DESC_JSON[key][0]
     except KeyError:
@@ -233,7 +232,7 @@ def get_desc_json_name(key: str) -> str:
 
 
 def get_desc_json_field_type(key: str) -> FieldTypeEnum:
-    """Get the FieldTypeEnum of a given key from the definition in repod.models.repo.DESC_JSON
+    """Get the FieldTypeEnum of a given key from the definition in repod.models.repo.DESC_JSON.
 
     Parameters
     ----------
@@ -250,27 +249,25 @@ def get_desc_json_field_type(key: str) -> FieldTypeEnum:
     str
         The FieldTypeEnum of a given 'desc' file identifier provided by key
     """
-
     try:
         return DESC_JSON[key][1]
     except KeyError:
         raise RepoManagementFileError(f"The key {key} is not a known 'desc' file identifier.")
 
 
-def get_files_json_keys() -> Set[str]:
-    """Get the keys of repod.models.repo.FILES_JSON
+def get_files_json_keys() -> set[str]:
+    """Get the keys of repod.models.repo.FILES_JSON.
 
     Returns
     -------
-    Set[str]
+    set[str]
         A set of strings representing the keys of repod.models.repo.FILES_JSON
     """
-
     return set(FILES_JSON.keys())
 
 
 def get_files_json_name(key: str) -> str:
-    """Get the JSON name of a given key from the definition in repod.models.repo.FILES_JSON
+    """Get the JSON name of a given key from the definition in repod.models.repo.FILES_JSON.
 
     Parameters
     ----------
@@ -287,7 +284,6 @@ def get_files_json_name(key: str) -> str:
     str
         The JSON name of a given 'files' file identifier provided by key
     """
-
     try:
         return FILES_JSON[key][0]
     except KeyError:
@@ -295,7 +291,7 @@ def get_files_json_name(key: str) -> str:
 
 
 def get_files_json_field_type(key: str) -> FieldTypeEnum:
-    """Get the FieldTypeEnum of a given key from the definition in repod.models.repo.FILES_JSON
+    """Get the FieldTypeEnum of a given key from the definition in repod.models.repo.FILES_JSON.
 
     Parameters
     ----------
@@ -312,7 +308,6 @@ def get_files_json_field_type(key: str) -> FieldTypeEnum:
     str
         The FieldTypeEnum of a given 'desc' file identifier provided by key
     """
-
     try:
         return FILES_JSON[key][1]
     except KeyError:
@@ -320,7 +315,7 @@ def get_files_json_field_type(key: str) -> FieldTypeEnum:
 
 
 class SyncDatabase(BaseModel):
-    """A model describing a repository sync database
+    """A model describing a repository sync database.
 
     Attributes
     ----------
@@ -333,8 +328,8 @@ class SyncDatabase(BaseModel):
     """
 
     database: Path
-    database_type: Optional[RepoDbTypeEnum]
-    compression_type: Optional[CompressionTypeEnum]
+    database_type: RepoDbTypeEnum | None = None
+    compression_type: CompressionTypeEnum | None = None
     desc_version: PackageDescVersionEnum
     files_version: FilesVersionEnum
 
@@ -342,12 +337,12 @@ class SyncDatabase(BaseModel):
     async def outputpackagebase_to_tarfile(
         cls,
         tarfile: TarFile,
-        database_type: Optional[RepoDbTypeEnum],
+        database_type: RepoDbTypeEnum | None,
         model: outputpackage.OutputPackageBase,
         packagedesc_version: PackageDescVersionEnum,
         files_version: FilesVersionEnum,
     ) -> None:
-        """Stream descriptor files derived from an OutputPackageBase to a TarFile
+        """Stream descriptor files derived from an OutputPackageBase to a TarFile.
 
         Allows streaming to a default repository database or a files database
 
@@ -364,8 +359,7 @@ class SyncDatabase(BaseModel):
         files_version: FilesVersionEnum
             The version of Files to use
         """
-
-        for (desc_model, files_model) in await model.get_packages_as_models(
+        for desc_model, files_model in await model.get_packages_as_models(
             packagedesc_version=packagedesc_version,
             files_version=files_version,
         ):
@@ -400,14 +394,13 @@ class SyncDatabase(BaseModel):
                 tarfile.addfile(files_file, io.BytesIO(files_content.getvalue().encode()))
 
     async def add(self, model: outputpackage.OutputPackageBase) -> None:
-        """Write descriptor files for packages of a single pkgbase to the repository sync database
+        """Write descriptor files for packages of a single pkgbase to the repository sync database.
 
         Parameters
         ----------
         model: OutputPackageBase
             The model to use for streaming descriptor files to the repository database
         """
-
         debug(f"Opening file {self.database} for writing...")
 
         with open_tarfile(
@@ -423,19 +416,17 @@ class SyncDatabase(BaseModel):
                 files_version=self.files_version,
             )
 
-    async def outputpackagebases(self) -> List[Tuple[str, outputpackage.OutputPackageBase]]:
-        """Read a repository database and yield the name of each pkgbase and the respective data (represented as an
-        instance of OutputPackageBase) in a Tuple.
+    async def outputpackagebases(self) -> list[tuple[str, outputpackage.OutputPackageBase]]:
+        """Read a repo sync database and return the name of each pkgbase and respective data.
 
         Returns
         -------
-        Iterator[Tuple[str, OutputPackageBase]]:
-            A Tuple holding the name of a pkgbase and its accompanying data in an instance of OutputPackageBase
+        list[tuple[str, OutputPackageBase]]:
+            A tuple holding the name of a pkgbase and its accompanying data in an instance of OutputPackageBase
         """
-
-        packages: Dict[str, outputpackage.OutputPackageBase] = {}
-        package_descs: Dict[str, PackageDesc] = {}
-        package_files: Dict[str, Files] = {}
+        packages: dict[str, outputpackage.OutputPackageBase] = {}
+        package_descs: dict[str, PackageDesc] = {}
+        package_files: dict[str, Files] = {}
 
         with open_tarfile(path=self.database, compression=self.compression_type) as db_file:
             for file_name in [
@@ -477,7 +468,7 @@ class SyncDatabase(BaseModel):
                         f"The database file {self.database} contains the member {file_name} of an unsupported type!"
                     )
 
-        for (name, package_desc) in package_descs.items():
+        for name, package_desc in package_descs.items():
             if packages.get(package_desc.get_base()):
                 packages[package_desc.get_base()].add_packages(
                     [package_desc.get_output_package(files=package_files.get(name))]
@@ -492,22 +483,16 @@ class SyncDatabase(BaseModel):
         return list(packages.items())
 
     async def stream_management_repo(self, path: Path) -> None:
-        """Stream descriptor files read from the JSON files of a management repository to the repository sync database
+        """Stream descriptor files read from JSON files of a management repository to the repository sync database.
 
         Parameters
         ----------
         path: Path
             The directory containing the files of the management repository
-
-        Raises
-        ------
-        RepoManagementFileNotFoundError
-            If no JSON files are found in path
         """
-
         file_list = sorted(path.glob("*.json"))
         if not file_list:
-            raise RepoManagementFileNotFoundError(f"There are no JSON files in {path}!")
+            debug(f"There are no JSON files in {path}! Creating empty sync db.")
 
         with open_tarfile(self.database, compression=self.compression_type, mode="w") as database_file:
             for json_file in file_list:
@@ -521,7 +506,7 @@ class SyncDatabase(BaseModel):
 
 
 class PackageDesc(BaseModel):
-    """A template class with helper methods to create instances of one of its (versioned) subclasses
+    """A template class with helper methods to create instances of one of its (versioned) subclasses.
 
     This class should not be instantiated directly, as it only provides generic instance methods for its subclasses.
 
@@ -529,14 +514,14 @@ class PackageDesc(BaseModel):
     """
 
     @classmethod
-    def from_dict(cls, data: Dict[str, Union[int, str, List[str]]]) -> PackageDesc:
-        """Create an instance of one of PackageDesc's subclasses from a dict
+    def from_dict(cls, data: dict[str, int | str | list[str]]) -> PackageDesc:
+        """Create an instance of one of PackageDesc's subclasses from a dict.
 
         This method should be used with data derived from reading a 'desc' file from a repository sync database.
 
         Parameters
         ----------
-        data: Dict[str, Union[int, str, List[str]]]
+        data: dict[str, int | str | list[str]]
             A dict containing data required to instantiate a subclass of PackageDesc
 
         Raises
@@ -551,25 +536,23 @@ class PackageDesc(BaseModel):
             An instance of one of the subclasses of PackageDesc
         """
 
-        def derive_package_desc_version(data: Set[str]) -> Optional[int]:
-            """Derive which PackageDesc subclass to instantiate
+        def derive_package_desc_version(data: set[str]) -> int | None:
+            """Derive which PackageDesc subclass to instantiate.
 
             Parameters
             ----------
-            data: Set[str]
+            data: set[str]
                  A set of strings representing the keys of the data dict passed to PackageDesc.from_dict()
 
             Returns
             -------
-            Optional[int]
+            int | None
                 The integer representing the version of the PackageDesc subclass, else None
             """
-
             for version, config in sorted(PACKAGE_DESC_VERSIONS.items(), reverse=True):
-
                 debug(f"Comparing 'desc' data to schema version {version}")
-                config_required: Set[str] = config["required"]  # type: ignore[assignment]
-                config_optional: Set[str] = config["optional"]  # type: ignore[assignment]
+                config_required: set[str] = config["required"]  # type: ignore[assignment]
+                config_optional: set[str] = config["optional"]  # type: ignore[assignment]
                 if config_required.issubset(data):
                     optionals = data - config_required
 
@@ -600,7 +583,7 @@ class PackageDesc(BaseModel):
         match detected_version:
             case 1:
                 try:
-                    return PackageDescV1(**data)
+                    return PackageDescV1.model_validate(data)
                 except ValidationError as e:
                     raise RepoManagementValidationError(
                         "A validation error occured while attempting to instantiate a PackageDescV1 using the data:\n"
@@ -608,7 +591,7 @@ class PackageDesc(BaseModel):
                     )
             case 2:
                 try:
-                    return PackageDescV2(**data)
+                    return PackageDescV2.model_validate(data)
                 except ValidationError as e:
                     raise RepoManagementValidationError(
                         "A validation error occured while attempting to instantiate a PackageDescV2 using the data:\n"
@@ -619,7 +602,7 @@ class PackageDesc(BaseModel):
 
     @classmethod
     async def from_stream(cls, data: io.StringIO) -> PackageDesc:
-        """Initialize a PackageDesc from an input stream
+        """Initialize a PackageDesc from an input stream.
 
         Parameters
         ----------
@@ -637,12 +620,11 @@ class PackageDesc(BaseModel):
         PackageDesc
             A PackageDesc instance based on data
         """
-
         current_header = ""
         current_type: FieldTypeEnum
-        int_types: Dict[str, int] = {}
-        string_types: Dict[str, str] = {}
-        string_list_types: Dict[str, List[str]] = {}
+        int_types: dict[str, int] = {}
+        string_types: dict[str, str] = {}
+        string_list_types: dict[str, list[str]] = {}
         keys = get_desc_json_keys()
 
         for line in data:
@@ -653,8 +635,8 @@ class PackageDesc(BaseModel):
             if line in keys:
                 current_header = get_desc_json_name(key=line)
                 current_type = get_desc_json_field_type(line)
-                # FIXME: find better way to provide a default (None or empty list for STRING_LIST as they all are
-                # Optional[List[str]]
+                # FIXME: find better way to provide a default
+                # (None or empty list for STRING_LIST as they all are list[str] | None)
                 if current_header and current_type == FieldTypeEnum.STRING_LIST:
                     string_list_types[current_header] = []
 
@@ -677,7 +659,7 @@ class PackageDesc(BaseModel):
                         f"\n{data.getvalue()}\n{e}"
                     )
 
-        desc_dict: Dict[str, Union[int, str, List[str]]] = {**int_types, **string_types, **string_list_types}
+        desc_dict: dict[str, int | str | list[str]] = {**int_types, **string_types, **string_list_types}
         try:
             return PackageDesc.from_dict(desc_dict)
         except RepoManagementValidationError as e:
@@ -686,7 +668,7 @@ class PackageDesc(BaseModel):
             )
 
     async def render(self, output: io.StringIO) -> None:
-        """Use a 'desc' jinja template to write the PackageDesc contents to an output stream
+        """Use a 'desc' jinja template to write the PackageDesc contents to an output stream.
 
         Parameters
         ----------
@@ -698,8 +680,10 @@ class PackageDesc(BaseModel):
         RepoManagementFileNotFoundError
             If no matching template can be found
         """
-
-        env = Environment(
+        # NOTE: We are not rendering HTML and need special characters, hence we are not affected by XSS problems and set
+        # autoescape=False
+        env = Environment(  # nosec: B701
+            autoescape=False,
             loader=PackageLoader("repod", "templates"),
             trim_blocks=True,
             lstrip_blocks=True,
@@ -713,17 +697,17 @@ class PackageDesc(BaseModel):
             template = env.get_template(template_file)
         except TemplateNotFound:
             raise RepoManagementFileNotFoundError(f"The 'desc' template file {template_file} could not be found!")
-        output.write(await template.render_async(self.dict()))
+        output.write(await template.render_async(self.model_dump(mode="json")))
 
-    def get_output_package(self, files: Optional[Files]) -> outputpackage.OutputPackage:
-        """Transform the PackageDesc model and an optional Files model into an OutputPackage model
+    def get_output_package(self, files: Files | None) -> outputpackage.OutputPackage:
+        """Transform the PackageDesc model and an optional Files model into an OutputPackage model.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the required fields!
         The PackageDesc class does not do that!
 
         Parameters
         ----------
-        files: Optional[Files]:
+        files: Files | None:
             A pydantic model, that represents the list of files, that belong to the package described by self
 
         Raises
@@ -740,10 +724,9 @@ class PackageDesc(BaseModel):
         OutputPackage
             A pydantic model, that describes a package and its list of files
         """
-
         schema_version = self.get_schema_version()
         schema_config = PACKAGE_DESC_VERSIONS.get(schema_version)
-        desc_dict = self.dict()
+        desc_dict = self.model_dump(mode="json")
         if schema_config:
             output_package_version = schema_config.get("output_package_version")
             match output_package_version:
@@ -751,7 +734,7 @@ class PackageDesc(BaseModel):
                     if files:
                         desc_dict["files"] = files
 
-                    return outputpackage.OutputPackageV1(**desc_dict)
+                    return outputpackage.OutputPackageV1.model_validate(desc_dict)
                 case _:
                     raise RuntimeError(
                         f"The OutputPackage version '{output_package_version}' is not valid. This is a bug!"
@@ -762,15 +745,15 @@ class PackageDesc(BaseModel):
                 "OutputPackage from a PackageDesc!"
             )
 
-    def get_output_package_base(self, files: Optional[Files]) -> outputpackage.OutputPackageBase:
-        """Transform the PackageDesc model and an Files model into an OutputPackageBase model
+    def get_output_package_base(self, files: Files | None) -> outputpackage.OutputPackageBase:
+        """Transform the PackageDesc model and an Files model into an OutputPackageBase model.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the required fields!
         The PackageDesc class does not do that!
 
         Parameters
         ----------
-        files: Optional[Files]:
+        files: Files | None:
             A pydantic model, that represents the list of files, that belong to the package described by self
 
         Raises
@@ -787,16 +770,15 @@ class PackageDesc(BaseModel):
         OutputPackageBase
             A pydantic model, that describes a package base and one of it's split packages
         """
-
         schema_version = self.get_schema_version()
         schema_config = PACKAGE_DESC_VERSIONS.get(schema_version)
-        desc_dict = self.dict()
+        desc_dict = self.model_dump(mode="json")
         if schema_config:
             output_package_version = schema_config.get("output_package_base_version")
             match output_package_version:
                 case 1:
                     desc_dict.update({"packages": [self.get_output_package(files=files)]})
-                    return outputpackage.OutputPackageBaseV1(**desc_dict)
+                    return outputpackage.OutputPackageBaseV1.model_validate(desc_dict)
                 case _:
                     raise RuntimeError(
                         f"The OutputPackageBase version '{output_package_version}' is not valid. This is a bug!"
@@ -808,7 +790,7 @@ class PackageDesc(BaseModel):
             )
 
     def get_base(self) -> str:
-        """Get the base attribute of the PackageDesc instance
+        """Get the base attribute of the PackageDesc instance.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the `base` field! The
         PackageDesc class does not do that!
@@ -823,16 +805,15 @@ class PackageDesc(BaseModel):
         str
             The base attribute of a PackageDesc subclass
         """
-
         if hasattr(self, "base"):
-            return str(self.base)  # type: ignore[attr-defined]
+            return str(self.base)
         else:
             raise RuntimeError(
                 "It is not possible to retrieve the 'base' attribute from the template class PackageDesc!"
             )
 
     def get_name(self) -> str:
-        """Get the name attribute of the PackageDesc instance
+        """Get the name attribute of the PackageDesc instance.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the `name` field! The
         PackageDesc class does not do that!
@@ -847,16 +828,15 @@ class PackageDesc(BaseModel):
         str
             The name attribute of a PackageDesc subclass
         """
-
         if hasattr(self, "name"):
-            return str(self.name)  # type: ignore[attr-defined]
+            return str(self.name)
         else:
             raise RuntimeError(
                 "It is not possible to retrieve the 'name' attribute from the template class PackageDesc!"
             )
 
     def get_schema_version(self) -> int:
-        """Get the schema_version attribute of the PackageDesc instance
+        """Get the schema_version attribute of the PackageDesc instance.
 
         NOTE: This method only successfully returns if the instance of the class using it defines the `schema_version`
         field! The PackageDesc class does not do that!
@@ -871,9 +851,8 @@ class PackageDesc(BaseModel):
         int
             The schema_version attribute of a PackageDesc subclass
         """
-
         if hasattr(self, "schema_version"):
-            return int(self.schema_version)  # type: ignore[attr-defined]
+            return int(self.schema_version)
         else:
             raise RuntimeError(
                 "It is not possible to retrieve the 'schema_version' attribute from the templatet class PackageDesc!"
@@ -881,7 +860,7 @@ class PackageDesc(BaseModel):
 
 
 class Files(BaseModel):
-    """A template class to describe files in the context of 'files' files in a repository sync database
+    """A template class to describe files in the context of 'files' files in a repository sync database.
 
     This class should not be instantiated directly, as it only provides generic instance methods for its subclasses.
 
@@ -889,14 +868,14 @@ class Files(BaseModel):
     """
 
     @classmethod
-    def from_dict(cls, data: Dict[str, List[str]]) -> Files:
-        """Class method to create one of Files' subclasses from a dict
+    def from_dict(cls, data: dict[str, list[str]]) -> Files:
+        """Class method to create one of Files' subclasses from a dict.
 
         This method is supposed to be called with data derived from a 'files' file.
 
         Parameters
         ----------
-        data: Dict[str, List[str]]
+        data: dict[str, list[str]]
             A dict with required data read from a 'files' file, which is used to instantiate one of Files' subclasses
 
         Raises
@@ -911,22 +890,20 @@ class Files(BaseModel):
             A Files instance (technically only one of its subclasses), instantiated from data
         """
 
-        def derive_files_version(data: Set[str]) -> Optional[int]:
-            """Derive which Files subclass to instantiate
+        def derive_files_version(data: set[str]) -> int | None:
+            """Derive which Files subclass to instantiate.
 
             Parameters
             ----------
-            data: Set[str]
+            data: set[str]
                  A set of strings representing the keys of the data dict passed to Files.from_dict()
 
             Returns
             -------
-            Optional[int]
+            int | None
                 The integer representing the schema version of the Files subclass, else None
             """
-
             for version, config in sorted(FILES_VERSIONS.items(), reverse=True):
-
                 debug(f"Comparing 'files' data to schema version {version}")
                 config_required = config["required"]
                 config_optional = config["optional"]
@@ -959,7 +936,7 @@ class Files(BaseModel):
         match detected_version:
             case 1:
                 try:
-                    return FilesV1(**data)
+                    return FilesV1.model_validate(data)
                 except ValidationError as e:
                     raise RepoManagementValidationError(
                         "A validation error occured while attempting to instantiate a FilesV1 using the data:\n"
@@ -972,7 +949,7 @@ class Files(BaseModel):
 
     @classmethod
     async def from_stream(cls, data: io.StringIO) -> Files:
-        """Initialize a Files from an input stream
+        """Initialize a Files from an input stream.
 
         Parameters
         ----------
@@ -989,10 +966,9 @@ class Files(BaseModel):
         Files
             A Files instance based on data
         """
-
         current_header = ""
         current_type: FieldTypeEnum
-        string_list_types: Dict[str, List[str]] = {}
+        string_list_types: dict[str, list[str]] = {}
         keys = get_files_json_keys()
 
         for line in data:
@@ -1019,7 +995,7 @@ class Files(BaseModel):
             )
 
     async def render(self, output: io.StringIO) -> None:
-        """Use a 'files' jinja template to write the Files contents to an output stream
+        """Use a 'files' jinja template to write the Files contents to an output stream.
 
         Parameters
         ----------
@@ -1031,8 +1007,9 @@ class Files(BaseModel):
         RepoManagementFileNotFoundError
             If no matching template can be found
         """
-
-        env = Environment(
+        # NOTE: We are not rendering HTML and need special characters, hence we are not affected by XSS problems and set
+        # autoescape=False
+        env = Environment(  # nosec: B701
             loader=PackageLoader("repod", "templates"),
             trim_blocks=True,
             lstrip_blocks=True,
@@ -1046,10 +1023,10 @@ class Files(BaseModel):
             template = env.get_template(template_file)
         except TemplateNotFound:
             raise RepoManagementFileNotFoundError(f"The 'files' template file {template_file} could not be found!")
-        output.write(await template.render_async(self.dict()))
+        output.write(await template.render_async(self.model_dump(mode="json")))
 
     def get_schema_version(self) -> int:
-        """Get the schema_version of the Files instance
+        """Get the schema_version of the Files instance.
 
         NOTE: This method only successfully returns if the instance of the class using it actually defines the
         `schema_version` field!
@@ -1064,9 +1041,8 @@ class Files(BaseModel):
         int
             The schema_version attribute of a Files subclass
         """
-
         if hasattr(self, "schema_version"):
-            return int(self.schema_version)  # type: ignore[attr-defined]
+            return int(self.schema_version)
         else:
             raise RuntimeError(
                 "It is not possible to retrieve the 'schema_version' attribute from the Files template class!"
@@ -1074,11 +1050,11 @@ class Files(BaseModel):
 
 
 class FilesV1(Files, FileList, SchemaVersionV1):
-    """A pydantic model to describe files in the context of 'files' files in a repository sync database (version 1)
+    """A pydantic model to describe files in the context of 'files' files in a repository sync database (version 1).
 
     Attributes
     ----------
-    files: Optional[List[str]]
+    files: list[str] | None
         An optional list of files. This is the data below a %FILES% identifier in a 'files' file, which identifies which
         file(s) belong to a package
     """
@@ -1114,14 +1090,14 @@ class PackageDescV1(
     Url,
     Version,
 ):
-    """A model describing all identifiers in a 'desc' file (version 1)
+    """A model describing all identifiers in a 'desc' file (version 1).
 
     Attributes
     ----------
     arch: str
         The attribute can be used to describe the (required) data below an %ARCH% identifier in a 'desc' file, which
         identifies a package's architecture
-    backup: Optional[List[str]]
+    backup: list[str] | None
         The attribute can be used to describe the (optional) data below a %BACKUP% identifier in a 'desc' file, which
         identifies which file(s) of a package pacman will create backups for
     base: str
@@ -1130,16 +1106,16 @@ class PackageDescV1(
     builddate: int
         The attribute can be used to describe the (required) data below a %BUILDDATE% identifier in a 'desc' file,
         which identifies a package's build date (represented in seconds since the epoch)
-    checkdepends: Optional[List[str]]
+    checkdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %CHECKDEPENDS% identifier in a 'desc' file,
         which identifies a package's checkdepends
-    conflicts: Optional[List[str]]
+    conflicts: list[str] | None
         The attribute can be used to describe the (optional) data below a %CONFLICTS% identifier in a 'desc' file, which
         identifies what other package(s) a package conflicts with
     csize: int
         The attribute can be used to describe the (required) data below a %CSIZE% identifier in a 'desc' file, which
         identifies a package's size
-    depends: Optional[List[str]]
+    depends: list[str] | None
         The attribute can be used to describe the (optional) data below a %DEPENDS% identifier in a 'desc' file, which
         identifies what other package(s) a package depends on
     desc: str
@@ -1148,16 +1124,16 @@ class PackageDescV1(
     filename: str
         The attribute can be used to describe the (required) data below a %FILENAME% identifier in a 'desc' file, which
         identifies a package's file name
-    groups: Optional[List[str]]
+    groups: list[str] | None
         The attribute can be used to describe the (optional) data below a %GROUPS% identifier in a 'desc' file, which
         identifies a package's groups
     isize: int
         The attribute can be used to describe the (required) data below an %ISIZE% identifier in a 'desc' file, which
         identifies a package's installed size
-    license: List[str]
+    license: list[str]
         The attribute can be used to describe the (required) data below a %LICENSE% identifier in a 'desc' file, which
         identifies a package's license(s)
-    makedepends: Optional[List[str]]
+    makedepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %MAKEDEPENDS% identifier in a 'desc' file,
         which identifies a package's makedepends
     md5sum: str
@@ -1166,7 +1142,7 @@ class PackageDescV1(
     name: str
         The attribute can be used to describe the (required) data below a %NAME% identifier in a 'desc' file, which
         identifies a package's name
-    optdepends: Optional[List[str]]
+    optdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %OPTDEPENDS% identifier in a 'desc' file,
         which identifies what other package(s) a package optionally depends on
     packager: str
@@ -1175,10 +1151,10 @@ class PackageDescV1(
     pgpsig: str
         The attribute can be used to describe the (optional) data below a %PGPSIG% identifier in a 'desc' file, which
         identifies a package's PGP signature
-    provides: Optional[List[str]]
+    provides: list[str] | None
         The attribute can be used to describe the (optional) data below a %PROVIDES% identifier in a 'desc' file, which
         identifies what other package(s) a package provides
-    replaces: Optional[List[str]]
+    replaces: list[str] | None
         The attribute can be used to describe the (optional) data below a %REPLACES% identifier in a 'desc' file, which
         identifies what other package(s) a package replaces
     schema_version: PositiveInt
@@ -1222,14 +1198,14 @@ class PackageDescV2(
     Url,
     Version,
 ):
-    """A model describing all identifiers in a 'desc' file (version 1)
+    """A model describing all identifiers in a 'desc' file (version 1).
 
     Attributes
     ----------
     arch: str
         The attribute can be used to describe the (required) data below an %ARCH% identifier in a 'desc' file, which
         identifies a package's architecture
-    backup: Optional[List[str]]
+    backup: list[str] | None
         The attribute can be used to describe the (optional) data below a %BACKUP% identifier in a 'desc' file, which
         identifies which file(s) of a package pacman will create backups for
     base: str
@@ -1238,16 +1214,16 @@ class PackageDescV2(
     builddate: int
         The attribute can be used to describe the (required) data below a %BUILDDATE% identifier in a 'desc' file,
         which identifies a package's build date (represented in seconds since the epoch)
-    checkdepends: Optional[List[str]]
+    checkdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %CHECKDEPENDS% identifier in a 'desc' file,
         which identifies a package's checkdepends
-    conflicts: Optional[List[str]]
+    conflicts: list[str] | None
         The attribute can be used to describe the (optional) data below a %CONFLICTS% identifier in a 'desc' file, which
         identifies what other package(s) a package conflicts with
     csize: int
         The attribute can be used to describe the (required) data below a %CSIZE% identifier in a 'desc' file, which
         identifies a package's size
-    depends: Optional[List[str]]
+    depends: list[str] | None
         The attribute can be used to describe the (optional) data below a %DEPENDS% identifier in a 'desc' file, which
         identifies what other package(s) a package depends on
     desc: str
@@ -1256,16 +1232,16 @@ class PackageDescV2(
     filename: str
         The attribute can be used to describe the (required) data below a %FILENAME% identifier in a 'desc' file, which
         identifies a package's file name
-    groups: Optional[List[str]]
+    groups: list[str] | None
         The attribute can be used to describe the (optional) data below a %GROUPS% identifier in a 'desc' file, which
         identifies a package's groups
     isize: int
         The attribute can be used to describe the (required) data below an %ISIZE% identifier in a 'desc' file, which
         identifies a package's installed size
-    license: List[str]
+    license: list[str]
         The attribute can be used to describe the (required) data below a %LICENSE% identifier in a 'desc' file, which
         identifies a package's license(s)
-    makedepends: Optional[List[str]]
+    makedepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %MAKEDEPENDS% identifier in a 'desc' file,
         which identifies a package's makedepends
     md5sum: str
@@ -1274,16 +1250,16 @@ class PackageDescV2(
     name: str
         The attribute can be used to describe the (required) data below a %NAME% identifier in a 'desc' file, which
         identifies a package's name
-    optdepends: Optional[List[str]]
+    optdepends: list[str] | None
         The attribute can be used to describe the (optional) data below a %OPTDEPENDS% identifier in a 'desc' file,
         which identifies what other package(s) a package optionally depends on
     packager: str
         The attribute can be used to describe the (required) data below a %PACKAGER% identifier in a 'desc' file, which
         identifies a package's packager
-    provides: Optional[List[str]]
+    provides: list[str] | None
         The attribute can be used to describe the (optional) data below a %PROVIDES% identifier in a 'desc' file, which
         identifies what other package(s) a package provides
-    replaces: Optional[List[str]]
+    replaces: list[str] | None
         The attribute can be used to describe the (optional) data below a %REPLACES% identifier in a 'desc' file, which
         identifies what other package(s) a package replaces
     schema_version: PositiveInt
@@ -1300,8 +1276,8 @@ class PackageDescV2(
     """
 
 
-def export_schemas(output: Union[Path, str]) -> None:
-    """Export the JSON schema of selected pydantic models to an output directory
+def export_schemas(output: Path | str) -> None:
+    """Export the JSON schema of selected pydantic models to an output directory.
 
     Parameters
     ----------
@@ -1313,7 +1289,6 @@ def export_schemas(output: Union[Path, str]) -> None:
     RuntimeError
         If output is not an existing directory
     """
-
     classes = [FilesV1, PackageDescV1, PackageDescV2]
 
     if isinstance(output, str):
@@ -1323,5 +1298,10 @@ def export_schemas(output: Union[Path, str]) -> None:
         raise RuntimeError(f"The output directory {output} must exist!")
 
     for class_ in classes:
-        with open(output / f"{class_.__name__}.json", "w") as f:
-            print(class_.schema_json(indent=2), file=f)
+        with open(output / f"{class_.__name__}.json", "wb") as f:
+            f.write(
+                dumps(
+                    class_.model_json_schema(),  # type: ignore[attr-defined]
+                    option=OPT_INDENT_2 | OPT_APPEND_NEWLINE | OPT_SORT_KEYS,
+                )
+            )
